/*
 * Copyright (C) 2009  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "aboutdlg.h"
#include "winmain.h"
#include "version.h"

#include <windows.h>
#include <shlobj.h>

using namespace plexdata::string;
using namespace plexdata::version;

/******************************************************************************
** CAboutDialog
*******************************************************************************/

#ifdef _WIN64
#define DLG_USR3264 DWLP_USER
#else 
#define DLG_USR3264 DWL_USER
#endif // _WIN64

CAboutDialog::CAboutDialog()
{
}

CAboutDialog::~CAboutDialog()
{
}

INT_PTR CAboutDialog::Show( HINSTANCE hInstance, HWND hParent )
{
	return ::DialogBoxParam
		( hInstance
		, MAKEINTRESOURCE( IDD_ABOUT )
		, hParent
		, (DLGPROC)CAboutDialog::DialogProc
		, (LPARAM)this 
		);
}

INT_PTR CALLBACK CAboutDialog::DialogProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	INT_PTR nResult = 0;
	CAboutDialog* pThis = reinterpret_cast<CAboutDialog*>( ::GetWindowLongPtr( hWnd, DLG_USR3264 ) );

	switch ( uMsg )
	{
		case WM_INITDIALOG:
		{
			// Get instance pointer.
			CAboutDialog* pThis = reinterpret_cast<CAboutDialog*>( lParam );

			// Initialize the dialog box.
			if ( NULL != pThis )
			{
				// Save instance pointer as window's user data.
				::SetWindowLongPtr( hWnd, DLG_USR3264, lParam );

				pThis->OnInitDialog( hWnd, wParam, lParam );

				nResult = TRUE; // Dialog box procedure should return TRUE ...
			}
			break;
		}
		case WM_COMMAND:
		{
			if ( NULL != pThis )
			{
				pThis->OnCommand( hWnd, wParam, lParam );
				nResult = 0; // Handled
			}
			break;
		}
	}

	return nResult;
}

void CAboutDialog::OnInitDialog( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
	CString      message;
	CVersionInfo version;

	// Set window icon
	::SendMessage( hWnd, WM_SETICON, (WPARAM)ICON_SMALL, (LPARAM)g_hIcon);

	// Initialize executable's version info.
	version.Initialize( g_strModule.String() );

	// Change current window caption.
	::SendMessage( hWnd, WM_GETTEXT, (WPARAM)100, (LPARAM)message.CreateBuffer(100) );
	message.CopyBuffer();
	message.ReleaseBuffer();
	message += version.InternalName();
	::SendMessage( hWnd, WM_SETTEXT, (WPARAM)0, (LPARAM)message.String() );


	// Change current window message.
	message.Format
#ifdef _WIN64
		( _T("%s (x64)\n\n%s\n\n%s %s\n\nVersion %s %s")
#else 
		( _T("%s\n\n%s\n\n%s %s\n\nVersion %s %s")
#endif // _WIN64
		, version.InternalName()
		, version.Comments()
		, version.LegalCopyright()
		, version.CompanyName()
		, version.FileVersion()
		, ( version.PrivateBuild() != NULL ) ? version.PrivateBuild() : _T("")
		);
	::SendDlgItemMessage( hWnd, IDLB_ABOUTTEXT, WM_SETTEXT, (WPARAM)0, (LPARAM)message.String() );

	// Send window into the front.
	::SetForegroundWindow( hWnd );
}

void CAboutDialog::OnCommand( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
	if ( BN_CLICKED == HIWORD( wParam ) )
	{
		if ( IDOK == LOWORD( wParam ) )
		{
			::EndDialog( hWnd, IDOK );
		}
		else if ( IDCANCEL == LOWORD( wParam ) )
		{
			::EndDialog( hWnd, IDCANCEL );
		}
	}
}

// EOF