/*
 * Copyright (C) 2009  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "config.h"
#include <stdio.h>		// Includes STL definitions.
#include <errno.h>		// Includes STL error codes.
#include <tchar.h>


LPCTSTR CConfig::SEC_GENERAL      = _T("tools");
LPCTSTR CConfig::ITEM_COUNT       = _T("count");      // Unsigned integer value.
LPCTSTR CConfig::SHORT_PATH       = _T("shortpath");  // RFU: Short path usage instead of long path.
LPCTSTR CConfig::ICON_SIZE        = _T("icons");      // Constant string value: SMALL or LARGE.
LPCTSTR CConfig::SYS_FONT         = _T("systemfont"); // Boolean value.

LPCTSTR CConfig::SEC_FONT         = _T("font");
LPCTSTR CConfig::FONT_NAME        = _T("name");       // String value.
LPCTSTR CConfig::FONT_SIZE        = _T("size");       // Integer value.
LPCTSTR CConfig::FONT_BOLD        = _T("bold");       // Boolean value.
LPCTSTR CConfig::FONT_ITALIC      = _T("italic");     // Boolean value.
LPCTSTR CConfig::FONT_UNDERLINE   = _T("underline");  // Boolean value.

LPCTSTR CConfig::SEC_ITEM         = _T("tool%lu");
LPCTSTR CConfig::ITEM_NAME        = _T("name");
LPCTSTR CConfig::ITEM_EXECUTABLE  = _T("path");
LPCTSTR CConfig::ITEM_SHOWCOMMAND = _T("show");
LPCTSTR CConfig::ITEM_PARAMETERS  = _T("param");
LPCTSTR CConfig::ITEM_WORKING     = _T("working");
LPCTSTR CConfig::ITEM_DESCRIPTION = _T("description");
LPCTSTR CConfig::ITEM_ICONFILE    = _T("iconfile");
LPCTSTR CConfig::ITEM_ICONINDEX   = _T("iconindex");

/********************************************************************************************
**
********************************************************************************************/
CConfig::CConfig()
{
}

/********************************************************************************************
**
********************************************************************************************/
CConfig::CConfig( LPCTSTR szFileName )
{
	this->SetFileName( szFileName );
}

/********************************************************************************************
**
********************************************************************************************/
CConfig::~CConfig()
{
}

/********************************************************************************************
**
********************************************************************************************/
bool CConfig::SetFileName( LPCTSTR szFileName )
{
	this->m_strFileName = szFileName;
	return ( !this->m_strFileName.IsEmpty() );
}

/********************************************************************************************
**
********************************************************************************************/
LPCTSTR CConfig::GetFileName()
{
	return this->m_strFileName.String();
}

/********************************************************************************************
**
********************************************************************************************/
bool CConfig::GetString( LPCTSTR szSection, LPCTSTR szVariable, LPTSTR pValue, size_t nValue )
{
	bool success = false;

	if ( ( NULL == szSection ) || ( _T('\0') == szSection[0] ) )
	{
		return success;
	}

	if ( ( NULL == szVariable ) || ( _T('\0') == szVariable[0] ) )
	{
		return success;
	}

	if ( ( NULL == pValue ) || ( nValue == 0 ) )
	{
		return success;
	}

	if ( this->m_strFileName.IsEmpty() )
	{
		return success;
	}

	size_t nTemp  = nValue + 1;
	TCHAR* szTemp = new TCHAR[nTemp];

	if ( NULL == szTemp )
	{
		return success;
	}

	size_t nResult = ::GetPrivateProfileString( szSection, szVariable, _T(""), szTemp, (unsigned long)nTemp, this->m_strFileName.String() );

	if ( ( 0 != nResult ) && ( nResult != ( nTemp - 1 ) ) )
	{
		success = ( 0 == _tcscpy_s( pValue, nValue, szTemp ) );
	}

	delete[] szTemp;

	return success;
}

/********************************************************************************************
**
********************************************************************************************/
bool CConfig::GetBoolean( LPCTSTR szSection, LPCTSTR szVariable, bool* pValue )
{
	bool success = false;

	if ( NULL == pValue )
	{
		return success;
	}

	TCHAR szTemp[20] = _T("");

	if ( this->GetString( szSection, szVariable, szTemp, sizeof(szTemp)/sizeof(TCHAR) ) )
	{
		if ( ( 0 == _tcsicmp( szTemp, _T("TRUE") ) ) || ( 0 == _tcsicmp( szTemp, _T("1") ) ) )
		{
			*pValue = true; // Set requested value.
			success = true;
		}
		else if ( ( 0 == _tcsicmp( szTemp, _T("FALSE") ) ) || ( 0 == _tcsicmp( szTemp, _T("0") ) ) )
		{
			*pValue = false; // Set requested value.
			success = true;
		}
		else
		{
			success = false;
		}
	}

	return success;
}

/********************************************************************************************
**
********************************************************************************************/
bool CConfig::GetSigned( LPCTSTR szSection, LPCTSTR szVariable, long* pValue )
{
	bool success = false;

	if ( NULL == pValue )
	{
		return success;
	}

	TCHAR szTemp[40] = _T("");

	if ( this->GetString( szSection, szVariable, szTemp, sizeof(szTemp)/sizeof(TCHAR) ) )
	{
		// Convert and set requested value.
		TCHAR** pDummy = (TCHAR**)NULL;
		*pValue = _tcstol( szTemp, pDummy, 10 );

		success = true;
	}

	return success;
}

/********************************************************************************************
**
********************************************************************************************/
bool CConfig::GetUnsigned( LPCTSTR szSection, LPCTSTR szVariable, unsigned long* pValue )
{
	bool success = false;

	if ( NULL == pValue )
	{
		return success;
	}

	TCHAR szTemp[40] = _T("");

	if ( this->GetString( szSection, szVariable, szTemp, sizeof(szTemp)/sizeof(TCHAR) ) )
	{
		// Convert and set requested value.
		TCHAR** pDummy = (TCHAR**)NULL;
		*pValue = _tcstoul( szTemp, pDummy, 10 );

		success = true;
	}

	return success;
}

/********************************************************************************************
**
********************************************************************************************/
bool CConfig::SetString( LPCTSTR szSection, LPCTSTR szVariable, LPCTSTR szValue, bool bQuoted )
{
	bool success = false;

	if ( ( NULL == szSection ) || ( _T('\0') == szSection[0] ) )
	{
		return success;
	}

	if ( ( NULL == szVariable ) || ( _T('\0') == szVariable[0] ) )
	{
		return success;
	}

	if ( ( NULL == szValue ) || ( _T('\0') == szValue[0] ) )
	{
		return success;
	}

	if ( this->m_strFileName.IsEmpty() )
	{
		return success;
	}

	size_t nTemp  = _tcslen( szValue ) + 3; // The value's size is the string length plus two for the quotes.
	TCHAR* szTemp = new TCHAR[nTemp];

	if ( NULL == szTemp )
	{
		return success;
	}

	// Prepare data to be written depending on quoting flag.
	if ( true == bQuoted )
	{
		_sntprintf_s( szTemp, nTemp-1, nTemp, _T("\"%s\""), szValue );
	}
	else
	{
		_tcscpy_s( szTemp, nTemp, szValue ); 
	}

	// Write data out to the configuration file and check the result.
	success = ( TRUE == ::WritePrivateProfileString( szSection, szVariable, szTemp, this->m_strFileName.String() ) );

	// Flush updated configuration file.
	::WritePrivateProfileString( NULL, NULL, NULL, this->m_strFileName.String() );

	delete[] szTemp;

	return success;
}

/********************************************************************************************
**
********************************************************************************************/
bool CConfig::SetBoolean( LPCTSTR szSection, LPCTSTR szVariable, bool bValue, bool bAsString )
{
	bool  success    = false;
	TCHAR szTemp[10] = _T("");

	if ( true == bAsString )
	{
		_tcscpy_s( szTemp, sizeof(szTemp)/sizeof(TCHAR), bValue ? _T("TRUE") : _T("FALSE") );
	}
	else
	{
		_tcscpy_s( szTemp, sizeof(szTemp)/sizeof(TCHAR), bValue ? _T("1") : _T("0") );
	}

	return this->SetString( szSection, szVariable, szTemp );
}

/********************************************************************************************
**
********************************************************************************************/
bool CConfig::SetSigned( LPCTSTR szSection, LPCTSTR szVariable, long lValue )
{
	TCHAR szTemp[20] = _T("");

	_sntprintf_s( szTemp, sizeof(szTemp)/sizeof(TCHAR)-1, sizeof(szTemp)/sizeof(TCHAR), _T("%ld"), lValue );

	return this->SetString( szSection, szVariable, szTemp );
}

/********************************************************************************************
**
********************************************************************************************/
bool CConfig::SetUnsigned( LPCTSTR szSection, LPCTSTR szVariable, unsigned long dwValue )
{
	TCHAR szTemp[20] = _T("");

	_sntprintf_s( szTemp, sizeof(szTemp)/sizeof(TCHAR)-1, sizeof(szTemp)/sizeof(TCHAR), _T("%lu"), dwValue );

	return this->SetString( szSection, szVariable, szTemp );
}

// EOF