/*
 * Copyright (C) 2009  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "helpdlg.h"

#include "winmain.h"
#include "splitpath.h"
#include "string.h"

#include <shlobj.h>

using namespace plexdata::string;

/******************************************************************************
** CHelpDialog
*******************************************************************************/

#ifdef _WIN64
#define DLG_USR3264 DWLP_USER
#else 
#define DLG_USR3264 DWL_USER
#endif // _WIN64

CHelpDialog::CHelpDialog()
{
}

CHelpDialog::~CHelpDialog()
{
}

INT_PTR CHelpDialog::Show( HINSTANCE hInstance, HWND hParent )
{
	return ::DialogBoxParam
		( hInstance
		, MAKEINTRESOURCE( IDD_HELP )
		, hParent
		, (DLGPROC)CHelpDialog::DialogProc
		, (LPARAM)this 
		);
}

INT_PTR CALLBACK CHelpDialog::DialogProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	INT_PTR nResult = 0;
	CHelpDialog* pThis = reinterpret_cast<CHelpDialog*>( ::GetWindowLongPtr( hWnd, DLG_USR3264 ) );

	switch ( uMsg )
	{
		case WM_INITDIALOG:
		{
			// Get instance pointer.
			CHelpDialog* pThis = reinterpret_cast<CHelpDialog*>( lParam );

			// Initialize the dialog box.
			if ( NULL != pThis )
			{
				// Save instance pointer as window's user data.
				::SetWindowLongPtr( hWnd, DLG_USR3264, lParam );

				pThis->OnInitDialog( hWnd, wParam, lParam );

				nResult = TRUE; // Dialog box procedure should return TRUE ...
			}
			break;
		}
		case WM_COMMAND:
		{
			if ( NULL != pThis )
			{
				pThis->OnCommand( hWnd, wParam, lParam );
				nResult = 0; // Handled
			}
			break;
		}
	}

	return nResult;
}

void CHelpDialog::OnInitDialog( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
	/** English param help version

	Usage:
		
		%s%s [option]

	Options:

		-run			Start this program, if it is not yet running.

		-install[:all]	Register this program within the Registry. If ":all" is used than
						register program for all users. Otherwise register program for
						current user only.

		-remove			Remove this program from the Registry.

		-helpfile		Display program's external help file.

		-help			Displays this help window.
	/**/

	// Set window icon
	::SendMessage( hWnd, WM_SETICON, (WPARAM)ICON_SMALL, (LPARAM)g_hIcon);

	// Display the help message box.
	CSplitPath splitter( g_strModule.String() );

	// Prepare executable's name
	TCHAR   szData[INFOTIPSIZE+1] = _T("");
	CString strData;
	::SendDlgItemMessage( hWnd, IDLB_PROGNAME, WM_GETTEXT, (WPARAM)INFOTIPSIZE, (LPARAM)szData );
	strData.Format( szData, splitter.Filename(), splitter.Extension() );
	::SendDlgItemMessage( hWnd, IDLB_PROGNAME, WM_SETTEXT, (WPARAM)0, (LPARAM)strData.String() );

	// Send window into the front.
	::SetForegroundWindow( hWnd );
}

void CHelpDialog::OnCommand( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
	if ( BN_CLICKED == HIWORD( wParam ) )
	{
		if ( IDOK == LOWORD( wParam ) )
		{
			::EndDialog( hWnd, IDOK );
		}
		else if ( IDCANCEL == LOWORD( wParam ) )
		{
			::EndDialog( hWnd, IDCANCEL );
		}
	}
}

// EOF