/*
 * Copyright (C) 2009  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "lvimove.h"

#include <crtdbg.h>

/******************************************************************************
** CAboutDialog
*******************************************************************************/

CListViewMoveItemsWrapper::CListViewMoveItemsWrapper( HWND hListView )
	: m_hListView( hListView )
{
	_ASSERT( this->m_hListView != NULL );
}

CListViewMoveItemsWrapper::~CListViewMoveItemsWrapper()
{
}

UINT CListViewMoveItemsWrapper::GetItemState( int iItem, UINT uMask )
{
	return ListView_GetItemState( this->m_hListView, iItem, uMask );
}

bool CListViewMoveItemsWrapper::GetItemText( int iItem, int iSubItem, LPTSTR& szText, int& nText )
{
	bool success = false;

	nText  = 255; // Most strings fitting that length.
	szText = new TCHAR[ nText + 1 ];

	if ( NULL != szText )
	{
		while ( true )
		{
			LVITEM theItem = {0};

			theItem.iItem      = iItem;
			theItem.iSubItem   = iSubItem;
			theItem.pszText    = szText;
			theItem.cchTextMax = nText;

			LRESULT lResult = ::SendMessage( this->m_hListView, LVM_GETITEMTEXT, (WPARAM)iItem, (LPARAM)&theItem );

			// Result string was truncated. We need another try.
			if ( lResult == theItem.cchTextMax - 1 )
			{
				delete[] szText;
				nText *= 2;
				szText = new TCHAR[ nText + 1 ];

				if ( NULL == szText )
				{
					break; // Error case!
				}
			}
			else
			{
				success = true;
				break;
			}
		}
	}

	return success;
}

void CListViewMoveItemsWrapper::SetItemText( int iItem, int iSubItem, LPCTSTR szText )
{
	_ASSERT( szText != NULL );
	ListView_SetItemText( this->m_hListView, iItem, iSubItem, const_cast<LPTSTR>( szText ) );
}

bool CListViewMoveItemsWrapper::GetItem( UINT uMask, UINT uStateMask, int iItem, int iSubItem, LPLVITEM pItem )
{
	_ASSERT( pItem != NULL );

	if ( uMask & LVIF_TEXT )
	{
		pItem->cchTextMax = 0;
		pItem->pszText    = NULL;

		if ( !this->GetItemText( iItem, iSubItem, pItem->pszText, pItem->cchTextMax ) )
		{
			return false;
		}

		uMask &= ~LVIF_TEXT;
	}

	pItem->mask      = uMask;
	pItem->stateMask = uStateMask;
	pItem->iItem     = iItem;
	pItem->iSubItem  = iSubItem;

	return ListView_GetItem( this->m_hListView, pItem ) ? true : false;
}

void CListViewMoveItemsWrapper::SwapItems( int iItem1, int iItem2 )
{
	const UINT ALLSTATES_MASK = (UINT)-1L;

	LVITEM lviLeft  = {0};
	LVITEM lviRight = {0};
	UINT   uMask    = LVIF_TEXT | LVIF_IMAGE | LVIF_INDENT | LVIF_PARAM  | LVIF_STATE;
	bool   bResult1 = this->GetItem( uMask, ALLSTATES_MASK, iItem1, 0, &lviLeft );
	bool   bResult2 = this->GetItem( uMask, ALLSTATES_MASK, iItem2, 0, &lviRight );

	if ( bResult1 && bResult2 )
	{
		lviLeft.iItem      = iItem2;
		lviLeft.mask       = uMask;
		lviLeft.stateMask  = ALLSTATES_MASK;

		lviRight.iItem     = iItem1;
		lviRight.mask      = uMask;
		lviRight.stateMask = ALLSTATES_MASK;

		// Swap the items
		ListView_SetItem( this->m_hListView, &lviLeft );
		ListView_SetItem( this->m_hListView, &lviRight );

		int iColumns = this->GetColumnsCount();

		//Loop for swapping each column in the items.
		for ( int iSubItem=1; iSubItem < iColumns; iSubItem++ )
		{
			// Its just fair to initialize variables.
			LPTSTR szText1 = NULL; int nText1 = 0;
			LPTSTR szText2 = NULL; int nText2 = 0;

			bResult1 = this->GetItemText( iItem1, iSubItem, szText1, nText1 );
			bResult2 = this->GetItemText( iItem2, iSubItem, szText2, nText2 );

			if ( bResult1 && bResult2 )
			{
				this->SetItemText( iItem2, iSubItem, szText1 );
				this->SetItemText( iItem1, iSubItem, szText2 );
			}

			delete[] szText1;
			delete[] szText2;
		}
	}

	delete[] lviLeft.pszText;
	delete[] lviRight.pszText;
}

//Move up the selected items
void CListViewMoveItemsWrapper::MoveSelectedItemsUp()
{
	int iCount = ListView_GetItemCount( this->m_hListView );
	int iFocus = -1L;

	for ( int iIndex=1; iIndex < iCount; iIndex++ )
	{
		if ( 0 != this->GetItemState( iIndex, LVIS_SELECTED ) )
		{
			if ( iFocus == -1L ) { iFocus = iIndex - 1; }

			this->SwapItems( iIndex, iIndex-1 );
		}
	}

	if ( iFocus != -1L )
	{
		ListView_EnsureVisible( this->m_hListView, iFocus, FALSE );
		ListView_SetItemState( this->m_hListView, iFocus, LVIS_FOCUSED | LVIS_SELECTED, LVIS_FOCUSED | LVIS_SELECTED );
		ListView_SetSelectionMark( this->m_hListView, iFocus );
	}
}

//Move down the selected items
void CListViewMoveItemsWrapper::MoveSelectedItemsDown()
{
	int iCount = ListView_GetItemCount( this->m_hListView );
	int iFocus = -1L;

	for ( int iIndex=iCount-1; iIndex >= 0; iIndex-- )
	{
		if ( 0 != this->GetItemState( iIndex, LVIS_SELECTED ) )
		{
			if ( iFocus == -1L ) { iFocus = iIndex + 1; }

			this->SwapItems( iIndex, iIndex + 1 );
		}
	}

	if ( iFocus != -1L )
	{
		ListView_EnsureVisible( this->m_hListView, iFocus, FALSE );
		ListView_SetItemState( this->m_hListView, iFocus, LVIS_FOCUSED | LVIS_SELECTED, LVIS_FOCUSED | LVIS_SELECTED );
		ListView_SetSelectionMark( this->m_hListView, iFocus );
	}
}

void CListViewMoveItemsWrapper::MoveSelectedItems( bool bDown )
{
	if ( bDown )
	{
		this->MoveSelectedItemsDown();
	}
	else
	{
		this->MoveSelectedItemsUp();
	}
}

// EOF