/*
 * Copyright (C) 2009  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "mandlg.h"
#include "winmain.h"
#include "aboutdlg.h"

#include <windows.h>
#include <htmlhelp.h>
#include <shlobj.h>

#include "regdlg.h"
#include "helpdlg.h"
#include "optdlg.h"
#include "version.h"
#include "lvimove.h"

using namespace plexdata::string;
using namespace plexdata::version;

/******************************************************************************
** CManipulateDialog
*******************************************************************************/

#ifdef _WIN64
#define DLG_USR3264 DWLP_USER
#define DLG_MSG3264 DWLP_MSGRESULT
#define GWL_WND3264 GWLP_WNDPROC
#else 
#define DLG_USR3264 DWL_USER
#define DLG_MSG3264 DWL_MSGRESULT
#define GWL_WND3264 GWL_WNDPROC
#endif // _WIN64

// Local helper function (no need to be a member function).
static bool IsKeyPressed( int nVirtKey )
{
	const short MASK_PRESSED = (short)0x8000;
	return ( ( ::GetKeyState( nVirtKey ) & MASK_PRESSED ) == MASK_PRESSED );
}

LONG_PTR CManipulateDialog::m_pLVWndProc = NULL;

CManipulateDialog::CManipulateDialog()
	: m_bOptionsChanged(false)
{
}

CManipulateDialog::~CManipulateDialog()
{
}

INT_PTR CManipulateDialog::Show( HINSTANCE hInstance, HWND hParent )
{
	return ::DialogBoxParam
		( hInstance
		, MAKEINTRESOURCE( IDD_MANAGE )
		, hParent
		, (DLGPROC)CManipulateDialog::DialogProc
		, (LPARAM)this 
		);
}

INT_PTR CALLBACK CManipulateDialog::DialogProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	INT_PTR nResult = 0;
	CManipulateDialog* pThis = reinterpret_cast<CManipulateDialog*>( ::GetWindowLongPtr( hWnd, DLG_USR3264 ) );

	switch ( uMsg )
	{
		case WM_INITDIALOG:
		{
			// Save the instance pointer.
			CManipulateDialog* pThis = reinterpret_cast<CManipulateDialog*>( lParam );

			// Initialize the dialog box.
			if ( NULL != pThis )
			{
				// Save instance pointer as window's user data.
				::SetWindowLongPtr( hWnd, DLG_USR3264, lParam );

				pThis->OnInitDialog( hWnd, wParam, lParam );

				nResult = TRUE; // Dialog box procedure should return TRUE ...
			}
			break;
		}
		case WM_SYSCOMMAND:
		{
			if ( NULL != pThis )
			{
				pThis->OnSysCommand( hWnd, wParam, lParam );
				nResult = 0; // Handled
			}
			break;
		}
		case WM_COMMAND:
		{
			if ( NULL != pThis )
			{
				pThis->OnCommand( hWnd, wParam, lParam );
				nResult = 0; // Handled
			}
			break;
		}
		case WM_NOTIFY:
		{
			if ( NULL != pThis )
			{
				pThis->OnNotify( hWnd, wParam, lParam );
				nResult = 0; // Return value is ignored.
			}
			break;
		}
		case WM_DROPFILES:
		{
			if ( NULL != pThis )
			{
				pThis->OnDropFiles( hWnd, wParam, lParam );
				nResult = 0; // Handled.
			}
			break;
		}
		case WM_DESTROY:
		{
			if ( NULL != pThis )
			{
				pThis->OnDestroy( hWnd, wParam, lParam );
				nResult = 0; // Handled.
			}
			break;
		}
	}

	return nResult;
}

void CManipulateDialog::OnInitDialog( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
	HWND     hList   = NULL; // Handle to the list view window.
	LVITEM   sItem   = { LVIF_TEXT | LVIF_PARAM, 0, 0, 0, 0, NULL, 0, 0, (LPARAM)NULL, 0 };
	LVCOLUMN sColumn = { 0, 0, 0, NULL, 0, 0, 0, 0 }; 
	DWORD    dwCount = 0L;
	DWORD    dwIndex = 0L;
	POINT    pPos    = {0};
	RECT     rWnd    = {0};
	RECT     rDsk    = {0};
	CRecord  cRecord;

	::SendMessage( hWnd, WM_SETICON, (WPARAM)ICON_SMALL, (LPARAM)g_hIcon);
	
	// Manipulate dialog's system menu.
	HMENU hSysMenu = ::GetSystemMenu( hWnd, FALSE );

	if ( NULL != hSysMenu )
	{
		int count = ::GetMenuItemCount( hSysMenu );
		for ( int index=0; index < count; index++ )
		{
			if ( SC_CLOSE == ::GetMenuItemID( hSysMenu, index ) )
			{
				MENUITEMINFO theItem = {0};
				theItem.cbSize = sizeof(MENUITEMINFO);
				theItem.fMask  = MIIM_TYPE | MIIM_ID;

				theItem.fType = MFT_SEPARATOR;
				::InsertMenuItem(hSysMenu,index,TRUE,&theItem);
				theItem.fType = MFT_STRING;

				theItem.wID        = WM_ABOUT_MENU;
				theItem.dwTypeData =  _T("ber ...");
				::InsertMenuItem(hSysMenu,index,TRUE,&theItem);

				theItem.wID        = WM_PARAMS_MENU;
				theItem.dwTypeData =  _T("Aufruf ...");
				::InsertMenuItem(hSysMenu,index,TRUE,&theItem);

				break;
			}
		}
	}

	// Initialize the list view.
	if ( NULL != ( hList = ::GetDlgItem( hWnd, IDC_DATALIST ) ) )
	{
		// Begin subclassing list view window.
		CManipulateDialog::m_pLVWndProc = ::SetWindowLongPtr( hList, GWL_WND3264, (LONG_PTR)CManipulateDialog::LVSubWndProc );

		ListView_SetExtendedListViewStyle( hList, LVS_EX_FULLROWSELECT );

		sColumn.mask     = LVCF_TEXT | LVCF_SUBITEM;
		sColumn.iSubItem = 0;
		sColumn.pszText  = _T("Programmname");
		ListView_InsertColumn( hList, 0, &sColumn );	

		sColumn.mask     = LVCF_TEXT | LVCF_SUBITEM;
		sColumn.iSubItem = 1;
		sColumn.pszText  = _T("Anzeige");
		ListView_InsertColumn( hList, 1, &sColumn );	

		sColumn.mask     = LVCF_TEXT | LVCF_SUBITEM;
		sColumn.iSubItem = 2;
		sColumn.pszText  = _T("Typ");
		ListView_InsertColumn( hList, 2, &sColumn );	

		// Set list data from the current configuration file.
		if ( g_Config.GetUnsigned( CConfig::SEC_GENERAL, CConfig::ITEM_COUNT, &dwCount ) )
		{
			for ( dwIndex=1L; dwIndex < dwCount+1; dwIndex++ )
			{
				TCHAR szData[INFOTIPSIZE+1] = _T("");
				long  nData = -1L;

				// Reset record data and initialize a new section name.
				cRecord.Reset();
				cRecord.Section( CConfig::SEC_ITEM, dwIndex );

				// Read data from configuration file and initialize depending record data.
				if ( !g_Config.GetString( cRecord.Section(), CConfig::ITEM_NAME, szData, INFOTIPSIZE ) )
				{
					continue;
				}
				else
				{
					cRecord.Name( szData );
				}

				if ( g_Config.GetString( cRecord.Section(), CConfig::ITEM_EXECUTABLE, szData, INFOTIPSIZE ) )
				{
					cRecord.Executable( szData );
				}

				if ( g_Config.GetSigned( cRecord.Section(), CConfig::ITEM_SHOWCOMMAND, &nData ) )
				{
					cRecord.ShowCommand( nData );
				}

				if ( g_Config.GetString( cRecord.Section(), CConfig::ITEM_PARAMETERS, szData, INFOTIPSIZE ) )
				{
					cRecord.Parameters( szData );
				}

				if ( g_Config.GetString( cRecord.Section(), CConfig::ITEM_WORKING, szData, INFOTIPSIZE ) )
				{
					cRecord.Working( szData );
				}

				if ( g_Config.GetString( cRecord.Section(), CConfig::ITEM_DESCRIPTION, szData, INFOTIPSIZE ) )
				{
					cRecord.Description( szData );
				}

				if ( g_Config.GetString( cRecord.Section(), CConfig::ITEM_ICONFILE, szData, INFOTIPSIZE ) )
				{
					cRecord.IconFile( szData );
				}

				if ( g_Config.GetSigned( cRecord.Section(), CConfig::ITEM_ICONINDEX, &nData ) )
				{
					cRecord.IconIndex( nData );
				}

				this->InsertItemAt( hList, &cRecord, dwIndex-1 );
			}
		}

		ListView_SetColumnWidth( hList, 0, LVSCW_AUTOSIZE_USEHEADER );
		ListView_SetColumnWidth( hList, 1, LVSCW_AUTOSIZE_USEHEADER );
		ListView_SetColumnWidth( hList, 2, LVSCW_AUTOSIZE_USEHEADER );

		// Select first list item.
		ListView_SetItemState( hList, 0, LVIS_FOCUSED|LVIS_SELECTED,LVIS_FOCUSED|LVIS_SELECTED);
		ListView_SetSelectionMark( hList, 0 );
	}

	// Get some size information.
	::GetWindowRect( hWnd, &rWnd );
	::SystemParametersInfo( SPI_GETWORKAREA, 0, &rDsk, 0 );

	// Calculate the window position.
	if ( ( 0 == rDsk.left ) && ( 0 == rDsk.top ) )
	{
		// Displays the window in the lower right corner. This happens 
		// when the Taskbar is either bottom or on the right site.
		pPos.x = rDsk.right  - ( ( rWnd.right - rWnd.left ) + 10 );
		pPos.y = rDsk.bottom - ( ( rWnd.bottom - rWnd.top ) + 10 );
	}
	else if ( ( 0 != rDsk.left ) && ( 0 == rDsk.top ) )
	{
		// Displays the window in the lower left corner. This happens 
		// when the Taskbar is on the left site.
		pPos.x = rDsk.left   + (                            + 10 );
		pPos.y = rDsk.bottom - ( ( rWnd.bottom - rWnd.top ) + 10 );
	}
	else if ( ( 0 == rDsk.left ) && ( 0 != rDsk.top ) )
	{
		// Displays the window in the upper right corner. This happens 
		// when the Taskbar is on the top site.
		pPos.x = rDsk.right - ( ( rWnd.right - rWnd.left ) + 10 );
		pPos.y = rDsk.top   + (                            + 10 );
	}

	// Move the window to the new position.
	::SetWindowPos( hWnd, NULL, pPos.x, pPos.y, 0, 0, SWP_NOSIZE );

	// Load images for up/down buttons.
	::SendDlgItemMessage
		( hWnd
		, IDBN_UP
		, BM_SETIMAGE
		, (WPARAM)IMAGE_ICON
		, (LPARAM)::LoadImage( g_hInstance, MAKEINTRESOURCE( IDI_UP ), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR )
		);

	::SendDlgItemMessage
		( hWnd
		, IDBN_DOWN
		, BM_SETIMAGE
		, (WPARAM)IMAGE_ICON
		, (LPARAM)::LoadImage( g_hInstance, MAKEINTRESOURCE( IDI_DOWN ), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR )
		);

	// Bring the window to the top of all other windows.
	::SetForegroundWindow( hWnd );
}

void CManipulateDialog::OnSysCommand( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
	if ( WM_ABOUT_MENU == wParam )
	{
		CAboutDialog dlg;
		dlg.Show( g_hInstance, hWnd );
	}
	else if ( WM_PARAMS_MENU == wParam )
	{
		CHelpDialog dlg;
		dlg.Show( g_hInstance, hWnd );
	}
}

void CManipulateDialog::OnCommand( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
	if ( BN_CLICKED == HIWORD( wParam ) )
	{
		if ( IDBN_SAVE == LOWORD( wParam ) )
		{
			// Save the result to the configuration file.
			this->SaveData( ::GetDlgItem( hWnd, IDC_DATALIST ) );
			::EndDialog( hWnd, IDOK );
		}
		else if ( IDCANCEL == LOWORD( wParam ) )
		{
			if ( m_bOptionsChanged )
			{
				CString strTitle;
				int nSize = ::GetWindowTextLength( hWnd ) + 1;
				::GetWindowText( hWnd, strTitle.CreateBuffer(nSize), nSize );
				strTitle.CopyBuffer();

				if ( IDYES == doQuestion( hWnd, strTitle.String(), _T("Soll die nderung der Optionen gespeichert werden?") ) )
				{
					this->SaveGlobalData( hWnd );
				}
			}

			::EndDialog( hWnd, IDCANCEL );
		}
		else if ( IDHELP == LOWORD( wParam ) )
		{
			// Help button was pressed!
			ShowHelp( hWnd );
		}
		else if ( IDBN_SEPARATOR == LOWORD( wParam ) )
		{
			CRecord cRecord;
			cRecord.Name( cRecord.Separator() );
			cRecord.ShowCommand( -1L );
			cRecord.IconIndex( -1L );
			// This rebuilds the section name!
			this->InsertItemAt( ::GetDlgItem( hWnd, IDC_DATALIST ), &cRecord, -1 );

		}
		else if ( IDBN_OPTIONS == LOWORD( wParam ) )
		{
			COptionDialog dlg;
			if ( IDOK == dlg.Show( g_hInstance, hWnd ) )
			{
				m_bOptionsChanged = true;
			}
		}
		else if ( IDBN_NEW == LOWORD( wParam ) )
		{
			CRegisterDialog dlg;

			if ( IDOK == dlg.Show( g_hInstance, hWnd, NULL, _T("Eintrag erstellen") ) )
			{
				// This rebuilds the section name!
				this->InsertItemAt( ::GetDlgItem( hWnd, IDC_DATALIST ), dlg.GetData(), -1 );
			}
		}
		else if ( IDBN_EDIT == LOWORD( wParam ) )
		{
			HWND   hList = ::GetDlgItem( hWnd, IDC_DATALIST );
			LVITEM sItem = { LVIF_PARAM, 0, 0, 0, 0, NULL, 0, 0, (LPARAM)NULL, 0 };

			sItem.iItem  = this->FirstSelected( hList );
			sItem.lParam = NULL;
		
			ListView_GetItem( hList, &sItem );

			CRegisterDialog dlg;
			
			if ( IDOK == dlg.Show( g_hInstance, hWnd, (CRecord*)sItem.lParam, _T("Eintrag bearbeiten") ) )
			{
				CRecord* pHelper = dlg.GetData();

				// This keeps original section name!
				((CRecord*)sItem.lParam)->Copy( pHelper, true );

				sItem.mask     = sItem.mask | LVIF_TEXT;
				sItem.iSubItem = 0;
				sItem.pszText  = (LPTSTR)pHelper->Name();
				ListView_SetItem( hList, &sItem );

				sItem.mask     = LVIF_TEXT;
				sItem.iSubItem = 1;
				sItem.pszText  = const_cast<LPTSTR>( pHelper->ShowString() );
				ListView_SetItem( hList, &sItem );
			}
		}
		else if ( IDBN_DELETE == LOWORD( wParam ) )
		{
			HWND   hList  = ::GetDlgItem( hWnd, IDC_DATALIST );
			LVITEM sItem  = { LVIF_PARAM, 0, 0, 0, 0, NULL, 0, 0, (LPARAM)NULL, 0 };
			int    iIndex = 0;

			while ( -1 != ( iIndex = ListView_GetNextItem( hList, -1, LVNI_SELECTED ) ) )
			{
				sItem.iItem  = iIndex;
				sItem.lParam = NULL;

				ListView_GetItem( ::GetDlgItem( hWnd, IDC_DATALIST ), &sItem );

				if ( NULL != sItem.lParam )
				{
					delete ( (CRecord*)sItem.lParam );
				}
						
				ListView_DeleteItem( hList, iIndex );
			}
		}
		else if ( IDBN_UP == LOWORD( wParam ) )
		{
			this->MoveItem( ::GetDlgItem( hWnd, IDC_DATALIST ), FALSE );
		}
		else if ( IDBN_DOWN == LOWORD( wParam ) )
		{
			this->MoveItem( ::GetDlgItem( hWnd, IDC_DATALIST ), TRUE );
		}
	}
}

void CManipulateDialog::OnNotify( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
	LPNMLISTVIEW pData = reinterpret_cast<LPNMLISTVIEW>( lParam );

	// Check input parameter.
	if ( NULL == pData ) { return; }

	// The sender was the data list.
	if ( IDC_DATALIST == (INT)wParam )
	{
		UINT iCount = ListView_GetSelectedCount( pData->hdr.hwndFrom );

		if ( LVN_ITEMCHANGED == pData->hdr.code )
		{
			::EnableWindow( ::GetDlgItem( hWnd, IDBN_UP   ), ( iCount != 0 ) && !this->IsListBegin( pData->hdr.hwndFrom ) );
			::EnableWindow( ::GetDlgItem( hWnd, IDBN_DOWN ), ( iCount != 0 ) && !this->IsListEnd( pData->hdr.hwndFrom )   );
			::EnableWindow( ::GetDlgItem( hWnd, IDBN_EDIT  ), ( iCount == 1 ) );
			::EnableWindow( ::GetDlgItem( hWnd, IDBN_DELETE  ), ( iCount != 0 ) );

			// Set focus to list view if nothing has the focus.
			if ( ::GetFocus() == NULL )
			{
				::SendMessage( hWnd, WM_NEXTDLGCTL, (WPARAM)pData->hdr.hwndFrom, 1 );
			}
    	}
		else if ( NM_DBLCLK == pData->hdr.code )
		{
			// Force creating of a new entry, if list has no selection.
			if ( iCount == 0 )
			{
				::SendMessage
					( hWnd
					, WM_COMMAND
					, MAKEWPARAM( IDBN_NEW, BN_CLICKED )
					, (LPARAM)::GetDlgItem( hWnd, IDBN_NEW ) 
					);
			}
			// Force editing of the currently selected entry.
			else if ( iCount == 1 )
			{
				::SendMessage
					( hWnd
					, WM_COMMAND
					, MAKEWPARAM( IDBN_EDIT, BN_CLICKED )
					, (LPARAM)::GetDlgItem( hWnd, IDBN_EDIT ) 
					);
			}
		}
		else if ( NM_RCLICK == pData->hdr.code )
		{
			HMENU        hMenu  = ::CreatePopupMenu();
			int          iMenu  = 0;
			POINT        sMouse = { 0, 0 };
			unsigned int iCount = ListView_GetSelectedCount( ::GetDlgItem( hWnd, IDC_DATALIST ) );

			// Get current tracking position.
			::GetCursorPos( &sMouse );

			// Display popup menu.
			if ( NULL != hMenu )
			{
				::AppendMenu( hMenu, MF_STRING,    100, _T("Neu")              );
				::AppendMenu( hMenu, MF_STRING,    200, _T("Bearbeiten")       );
				::AppendMenu( hMenu, MF_STRING,    300, _T("Lschen")          );
				::AppendMenu( hMenu, MF_SEPARATOR, 0,    NULL                  );
				::AppendMenu( hMenu, MF_STRING,    400, _T("Alles auswhlen")  );
				::AppendMenu( hMenu, MF_STRING,    500, _T("Auswahl aufheben") );

				// Disable items EDIT and DELETE.
				if ( iCount == 0 )
				{
					::EnableMenuItem( hMenu, 200, MF_GRAYED );
					::EnableMenuItem( hMenu, 300, MF_GRAYED );
				}
				// Disable items EDIT only.
				else if ( iCount != 1 )
				{
					::EnableMenuItem( hMenu, 200, MF_GRAYED );
				}

				// Track the popup menu now and destroy it after it.
				iMenu = ::TrackPopupMenu(
					hMenu, 
					TPM_RETURNCMD | TPM_LEFTBUTTON,
					sMouse.x, 
					sMouse.y, 
					0, 
					hWnd, 
					NULL 
				);

				::DestroyMenu( hMenu );

				// Check the popup result.
				switch ( iMenu )
				{
					case 100:	// Create new item.
					{
						::SendMessage
							( hWnd
							, WM_COMMAND
							, MAKEWPARAM( IDBN_NEW, BN_CLICKED )
							, (LPARAM)::GetDlgItem( hWnd, IDBN_NEW )
							);
						break;
					}
					case 200:	// Edit selected item.
					{
						::SendMessage
							( hWnd
							, WM_COMMAND
							, MAKEWPARAM( IDBN_EDIT, BN_CLICKED )
							, (LPARAM)::GetDlgItem( hWnd, IDBN_EDIT )
							);
						break;
					}
					case 300:	// Delete selected item(s).
					{
						::SendMessage
							( hWnd
							, WM_COMMAND
							, MAKEWPARAM( IDBN_DELETE, BN_CLICKED )
							, (LPARAM)::GetDlgItem( hWnd, IDBN_DELETE ) 
							);
						break;
					}
					case 400:	// Select all items.
					{
						ListView_SetItemState
							( ::GetDlgItem( hWnd, IDC_DATALIST )
							, -1
							, LVIS_SELECTED
							, LVIS_SELECTED 
							);
						break;
					}
					case 500:	// Unselect current selection.
					{
						ListView_SetItemState
							( ::GetDlgItem( hWnd, IDC_DATALIST )
							, -1
							, 0
							, LVIS_SELECTED 
							);
						break;
					}
				}
			}
		}
		else if ( LVN_KEYDOWN == pData->hdr.code )
		{
			switch ( ((LPNMLVKEYDOWN)pData)->wVKey )
			{
				// Support DELETE key!
				case VK_DELETE:
				{
					// At least one item is selected!
					if ( 0 < iCount )
					{
						// Save index of first selected item.
						int iSel = this->FirstSelected( pData->hdr.hwndFrom );

						// Delete all currently selected list items.
						::SendMessage
							( hWnd
							, WM_COMMAND
							, MAKEWPARAM( IDBN_DELETE, BN_CLICKED )
							, (LPARAM)::GetDlgItem( hWnd, IDBN_DELETE )
							);

						// Get new count of available list entries
						int iItems = ListView_GetItemCount( pData->hdr.hwndFrom );

						// Calculate index of new "last entry".
						if ( iItems <= iSel )
						{
							iSel = iItems - 1; 
						}

						// Select item directly behind currently deleted item(s).
						ListView_SetItemState
							( ::GetDlgItem( hWnd, IDC_DATALIST )
							, iSel
							, LVIS_SELECTED
							, LVIS_SELECTED
							);
					}
					break;
				}
			}
		}
	}
}

void CManipulateDialog::OnDropFiles( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
	HDROP hDrop = reinterpret_cast<HDROP>( wParam );

	if ( hDrop != NULL )
	{
		// Get the number of dropped files.
		UINT count = ::DragQueryFile( hDrop, -1L, NULL, 0 );

		for ( UINT index=0L; index < count; index++ )
		{
			// Get file name length.
			UINT nFile = ::DragQueryFile( hDrop, index, NULL, 0 ) + 1;

			// Create file name buffer.
			CString strFile;
			TCHAR* szFile = strFile.CreateBuffer( nFile );

			if ( szFile != NULL )
			{
				// Get file name.
				::DragQueryFile( hDrop, index, szFile, nFile );

				// Fill record with depending data.
				CRecord theRecord;
				if ( ResolveData( hWnd, szFile, theRecord ) )
				{
					// Append this new record.
					this->InsertItemAt( ::GetDlgItem( hWnd, IDC_DATALIST ), &theRecord, -1 );
				}
			}
		}
		::DragFinish( hDrop );
	}
}

void CManipulateDialog::OnDestroy( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
	HWND   hList   = NULL; // Handle to the list view window.
	LVITEM sItem   = { LVIF_PARAM, 0, 0, 0, 0, NULL, 0, 0, (LPARAM)NULL, 0 };
	DWORD  dwCount = 0L;
	DWORD  dwIndex = 0L;

	if ( NULL != ( hList = ::GetDlgItem( hWnd, IDC_DATALIST ) ) )
	{
		dwCount = ListView_GetItemCount( hList );

		for ( dwIndex=0L; dwIndex < dwCount; dwIndex++ )
		{
			sItem.iItem  = dwIndex;
			sItem.lParam = NULL;
		
			ListView_GetItem( hList, &sItem );

			if ( NULL != sItem.lParam )
			{
				delete (CRecord*)sItem.lParam;
			}
		}

		ListView_DeleteAllItems( hList );

		// End subclassing list view window.
		::SetWindowLongPtr( hList, GWL_WND3264, CManipulateDialog::m_pLVWndProc );
	}

	CloseHelp();
}

void CManipulateDialog::InsertItemAt( HWND hList, CRecord* pData, DWORD dwIndex )
{
	LVITEM sItem    = { LVIF_TEXT | LVIF_PARAM, 0, 0, 0, 0, NULL, 0, 0, (LPARAM)NULL, 0 };
	bool   bSection = true;	// Flag to determine handling of the given section entry.
	bool   bSelect  = false;

	// Check input parameters.
	if ( NULL == hList ) { return; }
	if ( NULL == pData ) { return; }

	// If given index is equal -1 (0xFFFFFFFF), then the record should be 
	// appended. In this case the new record should be selected too!
	if ( -1L == dwIndex ) // 0xFFFFFFFF
	{
		dwIndex  = ListView_GetItemCount( hList );
		bSection = false;
		bSelect  = true;
	}

	// Create a copy of given record.
	CRecord* pRecord = new CRecord( pData );
	if ( NULL == pRecord ) { return; }

	// Initialize section name if needed.
	if ( !bSection )
	{
		pRecord->Section( CConfig::SEC_ITEM, dwIndex );
	}

	// Append current record to the list (name only).
	sItem.mask     = LVIF_TEXT | LVIF_PARAM;
	sItem.iItem    = dwIndex;
	sItem.iSubItem = 0;
	sItem.pszText  = const_cast<LPTSTR>( pRecord->Name() );
	sItem.lParam   = reinterpret_cast<LPARAM>( pRecord );
	int iInsert = ListView_InsertItem( hList, &sItem );

	sItem.mask     = LVIF_TEXT;
	sItem.iItem    = dwIndex;
	sItem.iSubItem = 1;
	sItem.pszText  = const_cast<LPTSTR>( pRecord->ShowString() );
	sItem.lParam   = NULL;
	ListView_SetItem( hList, &sItem );

	CString strType; 
	GetRegFileType( pRecord->Executable(), strType );
	sItem.mask     = LVIF_TEXT;
	sItem.iItem    = dwIndex;
	sItem.iSubItem = 2;
	sItem.pszText  = const_cast<LPTSTR>( strType.String() );
	sItem.lParam   = NULL;
	ListView_SetItem( hList, &sItem );

	if ( bSelect )
	{
		// Un-select all selected items.
		ListView_SetItemState( hList, -1, 0, LVIS_SELECTED | LVIS_FOCUSED );

		// Select the new item.
		ListView_EnsureVisible( hList, iInsert, FALSE );
		ListView_SetItemState( hList, iInsert, LVIS_SELECTED | LVIS_FOCUSED, LVIS_SELECTED | LVIS_FOCUSED );
	}

	ListView_SetColumnWidth( hList, 0, LVSCW_AUTOSIZE );
}

void CManipulateDialog::SaveData( HWND hList )
{
	LVITEM sItem   = { LVIF_PARAM, 0, 0, 0, 0, NULL, 0, 0, (LPARAM)NULL, 0 };
	DWORD  dwCount = 0L;
	DWORD  dwIndex = 0L;

	// Check input parameter.
	if ( NULL == hList ) { return; }

	// Remove the old configuration file.
	::DeleteFile( g_Config.GetFileName() );

	// Get number of list entries.
	dwCount = ListView_GetItemCount( hList );

	// Set general section data.
	g_Config.SetUnsigned( CConfig::SEC_GENERAL, CConfig::ITEM_COUNT, dwCount );
	this->SaveGlobalData( ::GetParent( hList ) );

	// Put all tool sections into the sonfig file.
	for ( dwIndex=0L; dwIndex < dwCount; dwIndex++ )
	{
		sItem.iItem  = dwIndex;
		sItem.lParam = NULL;

		ListView_GetItem( hList, &sItem );

		if ( NULL != sItem.lParam )
		{
			// The section name is not yet set! Set it.
			((CRecord*)sItem.lParam)->Section( CConfig::SEC_ITEM, dwIndex + 1 );

			// Write new data out to the configuration file.
			g_Config.SetString( ((CRecord*)sItem.lParam)->Section(), CConfig::ITEM_NAME,        ((CRecord*)sItem.lParam)->Name()        );
			g_Config.SetString( ((CRecord*)sItem.lParam)->Section(), CConfig::ITEM_EXECUTABLE,  ((CRecord*)sItem.lParam)->Executable()  );
			g_Config.SetSigned( ((CRecord*)sItem.lParam)->Section(), CConfig::ITEM_SHOWCOMMAND, ((CRecord*)sItem.lParam)->ShowCommand() );
			g_Config.SetString( ((CRecord*)sItem.lParam)->Section(), CConfig::ITEM_PARAMETERS,  ((CRecord*)sItem.lParam)->Parameters()  );
			g_Config.SetString( ((CRecord*)sItem.lParam)->Section(), CConfig::ITEM_WORKING,     ((CRecord*)sItem.lParam)->Working()     );
			g_Config.SetString( ((CRecord*)sItem.lParam)->Section(), CConfig::ITEM_DESCRIPTION, ((CRecord*)sItem.lParam)->Description() );
			g_Config.SetString( ((CRecord*)sItem.lParam)->Section(), CConfig::ITEM_ICONFILE,    ((CRecord*)sItem.lParam)->IconFile()    );
			g_Config.SetSigned( ((CRecord*)sItem.lParam)->Section(), CConfig::ITEM_ICONINDEX,   ((CRecord*)sItem.lParam)->IconIndex()   );
		}
	}
}

void CManipulateDialog::SaveGlobalData( HWND hWnd )
{
	// IMPORTANT: If more global data have to be saved then put them here.

	_ASSERT( hWnd != NULL );

	HDC hDC = ::GetDC( hWnd );

	g_Config.SetBoolean ( CConfig::SEC_GENERAL, CConfig::SHORT_PATH, g_bShortPath );
	g_Config.SetBoolean ( CConfig::SEC_GENERAL, CConfig::SYS_FONT,   g_bSystemFont );
	g_Config.SetString  ( CConfig::SEC_GENERAL, CConfig::ICON_SIZE,  g_bLargeIcons ? _T("large") :  _T("small") );

	g_Config.SetString ( CConfig::SEC_FONT, CConfig::FONT_NAME,      g_lfMenuFont.lfFaceName );
	g_Config.SetSigned ( CConfig::SEC_FONT, CConfig::FONT_SIZE,      -::MulDiv( g_lfMenuFont.lfHeight, 72, ::GetDeviceCaps( hDC, LOGPIXELSY ) ) );
	g_Config.SetBoolean( CConfig::SEC_FONT, CConfig::FONT_BOLD,      ( g_lfMenuFont.lfWeight    == FW_BOLD ) ? true : false );
	g_Config.SetBoolean( CConfig::SEC_FONT, CConfig::FONT_ITALIC,    ( g_lfMenuFont.lfItalic    == TRUE    ) ? true : false );
	g_Config.SetBoolean( CConfig::SEC_FONT, CConfig::FONT_UNDERLINE, ( g_lfMenuFont.lfUnderline == TRUE    ) ? true : false );

	// Cleanup.
	::ReleaseDC( hWnd, hDC );
}

void CManipulateDialog::MoveItem( HWND hList, bool bDown )
{
	// Do not execute if list begin has been reached.
	if ( !bDown && this->IsListBegin( hList )  )
	{
		return;
	}

	// Do not execute if list end has been reached.
	if ( bDown && this->IsListEnd( hList ) )
	{
		return;
	}

	CListViewMoveItemsWrapper move( hList );
	
	move.MoveSelectedItems( bDown );
}

int CManipulateDialog::FirstSelected( HWND hList )
{
	int          iResult = -1;
	int          iCount  = 0;
	int          iIndex  = 0;
	unsigned int uiState = 0;

	// Check input parameters.
	if ( NULL == hList ) { return ( iResult ); }

	// Find the index of the currently selected item.
	iCount = ListView_GetItemCount( hList );
	for ( iIndex=0; iIndex < iCount; iIndex++ )
	{
		// Get the state of the current list item.
		uiState = ListView_GetItemState( hList, iIndex, LVIS_SELECTED );

		// If the first selected list item was found, break this loop.
		if ( uiState & LVIS_SELECTED )
		{
			iResult = iIndex;
			break;
		}
	}

	return iResult;
}

int CManipulateDialog::LastSelected( HWND hList )
{
	int          iResult = -1;
	int          iCount  = 0;
	int          iIndex  = 0;
	unsigned int uiState = 0;

	// Check input parameters.
	if ( NULL == hList ) { return ( iResult ); }

	// Find the index of the currently selected item.
	iCount = ListView_GetItemCount( hList );
	for ( iIndex=iCount-1; iIndex >= 0; iIndex-- )
	{
		// Get the state of the current list item.
		uiState = ListView_GetItemState( hList, iIndex, LVIS_SELECTED );

		// If the first selected list item was found, break this loop.
		if ( uiState & LVIS_SELECTED )
		{
			iResult = iIndex;
			break;
		}
	}

	return iResult;
}

bool CManipulateDialog::IsListBegin( HWND hList )
{
	return ( 0 == this->FirstSelected( hList ) );
}

bool CManipulateDialog::IsListEnd( HWND hList )
{
	return ( this->LastSelected( hList ) == ListView_GetItemCount( hList ) - 1 );
}

LRESULT CALLBACK CManipulateDialog::LVSubWndProc( HWND hList, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	CManipulateDialog* pThis = reinterpret_cast<CManipulateDialog*>( ::GetWindowLongPtr( ::GetParent( hList ), DLG_USR3264 ) );

	if ( pThis != NULL )
	{
		switch ( uMsg )
		{
			case WM_GETDLGCODE:
			{
				if ( wParam == VK_RETURN )
				{
					HWND hDlg  = ::GetParent( hList );
					HWND hEdit = ::GetDlgItem( hDlg, IDBN_EDIT );

					if ( ::IsWindowEnabled( hEdit ) )
					{
						::SendMessage( hDlg, WM_COMMAND, MAKEWPARAM( IDBN_EDIT, BN_CLICKED ), (LPARAM)hEdit );
					}
				}
				break;
			}
			case WM_KEYDOWN:
			{
				// Proceed only with control key!
				if ( IsKeyPressed( VK_CONTROL ) && !IsKeyPressed( VK_SHIFT ) )
				{
					// REMARK: Under that conditions for the list view it seems the CTRL-key would never be pressed.

					switch ( wParam )
					{
					case VK_UP:
						
						// Move item (same as corresponding button press)
						pThis->MoveItem( hList, false );

						break;

					case VK_DOWN:

						// Move item (same as corresponding button press)
						pThis->MoveItem( hList, true );

						break;
					}

					// Set dialog box result to "WM_KEYDOWN was handled"!
					::SetWindowLongPtr( ::GetParent( hList ), DLG_MSG3264, TRUE );

					return 0; // Message processed.
				}
				break;
			}
			case WM_KEYUP:
			{
				// Proceed only with control key!
				if ( IsKeyPressed( VK_CONTROL ) && !IsKeyPressed( VK_SHIFT ) )
				{
					// REMARK: Under that conditions for the list view it seems the CTRL-key would never be pressed.

					// Set dialog box result to "WM_KEYUP was handled"!
					::SetWindowLongPtr( ::GetParent( hList ), DLG_MSG3264, TRUE );

					return 0; // Message processed.
				}
				break;
			}
		}
	}

	// Call original window procedure for any unhandled message.
	return ::CallWindowProc( (WNDPROC)CManipulateDialog::m_pLVWndProc, hList, uMsg, wParam, lParam );
}

// EOF