/*
 * Copyright (C) 2009  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "msghelp.h"

#include <tchar.h>
#include <stdio.h>

#define TITLE_ERROR    _T("Fehler")			// The error title extension.
#define TITLE_MESSAGE  _T("Nachricht")		// The message title extension.
#define TITLE_WARNING  _T("Warnung")		// The warning title extension.
#define TITLE_QUESTION _T("Frage")			// The warning title extension.

TCHAR szBoxTitle[64];	// A buffer to display the message box title.
TCHAR szBoxMsg[3072];	// A buffer for the displayed message.

void doError( HWND hWnd, DWORD dwErr, LPCTSTR szTitle, LPCTSTR szFormat, ...  )
{
	TCHAR   szSystem[1024] = _T("");// A buffer to format the system message.
	DWORD   dwFlags        = 0;		// A flag list for function 'FormatMessage()'.
	DWORD   dwLangId       = 0;		// The used language for function 'FormatMessage()'.
	va_list pArgs;					// The argument list.
	double  dDummy         = 0.0;	// Needed to load floating point library only.
									// Otherwise the runtime error R6002 will occur!!!

	// To avoid to display an error box with message 'no error'.
	if ( ERROR_SUCCESS == dwErr ) { return; }

	// Reset the message box message and title.
	szBoxTitle[0] = _T('\0');
	szBoxMsg[0]   = _T('\0');

	// Prepare a new message box title.
	if ( NULL != szTitle  )
	{
		_tcsncpy_s( szBoxTitle, sizeof(szBoxTitle)/sizeof(TCHAR), szTitle, _tcslen( szTitle ) );
	}
	else
	{
		_tcsncpy_s( szBoxTitle, sizeof(szBoxTitle)/sizeof(TCHAR), TITLE_ERROR, _tcslen( TITLE_ERROR ) );
	}

	// Prepare the core message string.
	if ( NULL != szFormat )
	{
		// Format the given parameter list.
		va_start( pArgs, szFormat );
		_vsntprintf_s( szBoxMsg, sizeof(szBoxMsg)/sizeof(TCHAR), sizeof(szBoxMsg)/sizeof(TCHAR)-1, szFormat, pArgs );
		va_end( pArgs );
	}

	// If the given error code is not '-1', then get a formatted system message.
	if ( -1 != dwErr )
	{
		// Initialize needed flags.
		dwFlags  = FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS;
		dwLangId = MAKELANGID( LANG_NEUTRAL, SUBLANG_DEFAULT );

		// Get a formatted message from the system
		::FormatMessage( dwFlags, NULL, dwErr, dwLangId, (LPTSTR)szSystem, sizeof(szSystem)/sizeof(TCHAR), NULL );
	}
	
	// Prepare the display message.
	if ( _T('\0') != szBoxMsg[0] )
	{
		_tcsncat_s( szBoxMsg, sizeof(szBoxMsg)/sizeof(TCHAR), _T("\n"), _tcslen( _T("\n") ) );
	}

	if ( _T('\0') != szSystem[0] )
	{
		_tcsncat_s( szBoxMsg, sizeof(szBoxMsg)/sizeof(TCHAR), szSystem, _tcslen( szSystem ) );
	}

	// Display valid messages only.
	if ( _T('\0') != szBoxMsg[0] )
	{
		// Display an error message box with the currently formatted message.
		::MessageBox( hWnd, szBoxMsg, szBoxTitle, MB_OK|MB_ICONERROR );
	}
}

void doMessage( HWND hWnd, LPCTSTR szTitle, LPCTSTR szFormat, ...  )
{
	va_list pArgs;			// The argument list.
	double  dDummy  = 0.0;	// Needed to load floating point library only.
							// Otherwise the runtime error R6002 will occur!!!

	// Reset the message box message and title.
	szBoxTitle[0] = _T('\0');
	szBoxMsg[0]   = _T('\0');

	// Prepare a new message box title.
	if ( NULL != szTitle  )
	{
		_tcsncpy_s( szBoxTitle, sizeof(szBoxTitle)/sizeof(TCHAR), szTitle, _tcslen( szTitle ) );
	}
	else
	{
		_tcsncpy_s( szBoxTitle, sizeof(szBoxTitle)/sizeof(TCHAR), TITLE_MESSAGE, _tcslen( TITLE_MESSAGE ) );
	}

	// Prepare the core message string.
	if ( NULL != szFormat )
	{
		// Format the given parameter list.
		va_start( pArgs, szFormat );
		_vsntprintf_s( szBoxMsg, sizeof(szBoxMsg)/sizeof(TCHAR), sizeof(szBoxMsg)/sizeof(TCHAR)-1, szFormat, pArgs );
		va_end( pArgs );
	}

	// Display valid messages only.
	if ( _T('\0') != szBoxMsg[0] )
	{
		// Display a message box with the currently formatted message.
		::MessageBox( hWnd, szBoxMsg, szBoxTitle, MB_OK|MB_ICONINFORMATION );
	}
}

void doWarning( HWND hWnd, LPCTSTR szTitle, LPCTSTR szFormat, ...  )
{
	va_list pArgs;			// The argument list.
	double  dDummy  = 0.0;	// Needed to load floating point library only.
							// Otherwise the runtime error R6002 will occur!!!

	// Reset the message box message and title.
	szBoxTitle[0] = _T('\0');
	szBoxMsg[0]   = _T('\0');

	// Prepare a new message box title.
	if ( NULL != szTitle  )
	{
		_tcsncpy_s( szBoxTitle, sizeof(szBoxTitle)/sizeof(TCHAR), szTitle, _tcslen( szTitle ) );
	}
	else
	{
		_tcsncpy_s( szBoxTitle, sizeof(szBoxTitle)/sizeof(TCHAR), TITLE_WARNING, _tcslen( TITLE_WARNING ) );
	}

	// Prepare the core message string.
	if ( NULL != szFormat )
	{
		// Format the given parameter list.
		va_start( pArgs, szFormat );
		_vsntprintf_s( szBoxMsg, sizeof(szBoxMsg)/sizeof(TCHAR), sizeof(szBoxMsg)/sizeof(TCHAR)-1, szFormat, pArgs );
		va_end( pArgs );
	}

	// Display valid messages only.
	if ( _T('\0') != szBoxMsg[0] )
	{
		// Display a warning message box with the currently formatted message.
		::MessageBox( hWnd, szBoxMsg, szBoxTitle, MB_OK|MB_ICONWARNING );
	}
}

int	doQuestion( HWND hWnd, LPCTSTR szTitle, LPCTSTR szFormat, ...  )
{
	va_list pArgs;			// The argument list.
	double  dDummy  = 0.0;	// Needed to load floating point library only.
							// Otherwise the runtime error R6002 will occur!!!

	// Reset the message box message and title.
	szBoxTitle[0] = _T('\0');
	szBoxMsg[0]   = _T('\0');

	// Prepare a new message box title.
	if ( NULL != szTitle  )
	{
		_tcsncpy_s( szBoxTitle, sizeof(szBoxTitle)/sizeof(TCHAR), szTitle, _tcslen( szTitle ) );
	}
	else
	{
		_tcsncpy_s( szBoxTitle, sizeof(szBoxTitle)/sizeof(TCHAR), TITLE_QUESTION, _tcslen( TITLE_QUESTION ) );
	}

	// Prepare the core message string.
	if ( NULL != szFormat )
	{
		// Format the given parameter list.
		va_start( pArgs, szFormat );
		_vsntprintf_s( szBoxMsg, sizeof(szBoxMsg)/sizeof(TCHAR), sizeof(szBoxMsg)/sizeof(TCHAR)-1, szFormat, pArgs );
		va_end( pArgs );
	}

	// Display valid messages only.
	if ( _T('\0') != szBoxMsg[0] )
	{
		// Display a question message box with the currently formatted message.
		return ::MessageBox( hWnd, szBoxMsg, szBoxTitle, MB_YESNO|MB_ICONQUESTION );
	}
	else
	{
		return IDABORT;
	}
}

#ifdef _DEBUG
#include <logapi.h>
#ifdef _WIN64
#pragma comment( lib, "logapi64d.lib" )
#else
#pragma comment( lib, "logapid.lib" )
#endif // _WIN64
#endif // _DEBUG

void doLogging( LPCTSTR szFormat, ...  )
{
#ifdef _DEBUG
	static HANDLE hLogger = NULL;

	if ( hLogger == NULL )
	{
		hLogger = ::LogAttach( LOGTYPE_CONSOLE, false, false, false, _T("TrayLaunch - Logger") );
		::LogResize( hLogger, 1000, 500 );
	}

	TCHAR*  output  = NULL;
	size_t  length  = 0;
	double  dummy   = 0.0; // Load floating point library, otherwise runtime error R6002 will occur!
	va_list argList;

	if ( szFormat != NULL )
	{
		va_start( argList, szFormat );
		length = _vsctprintf( szFormat, argList ) + 1;
		if ( length > 0 && NULL != ( output = new TCHAR[length + 1] ) )
		{
			if ( 0 < _vstprintf_s( output, length, szFormat, argList ) )
			{
				::LogMessage( hLogger, output );
			}
		}
		va_end( argList );
	}
	// Unload in a DEBUG version? Don't care!
#endif // _DEBUG
}

// EOF