/*
 * Copyright (C) 2009  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "optdlg.h"

#include "winmain.h"
#include "string.h"

#include <shlobj.h>

using namespace plexdata::string;

/******************************************************************************
** COptionDialog
*******************************************************************************/

#ifdef _WIN64
#define DLG_USR3264 DWLP_USER
#else 
#define DLG_USR3264 DWL_USER
#endif // _WIN64

COptionDialog::COptionDialog()
	: m_hSampleFont(NULL)
{
}

COptionDialog::~COptionDialog()
{
	::DeleteObject( this->m_hSampleFont );
}

INT_PTR COptionDialog::Show( HINSTANCE hInstance, HWND hParent )
{
	return ::DialogBoxParam
		( hInstance
		, MAKEINTRESOURCE( IDD_OPTIONS )
		, hParent
		, (DLGPROC)COptionDialog::DialogProc
		, (LPARAM)this 
		);
}

INT_PTR CALLBACK COptionDialog::DialogProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	INT_PTR nResult = 0;
	COptionDialog* pThis = reinterpret_cast<COptionDialog*>( ::GetWindowLongPtr( hWnd, DLG_USR3264 ) );

	switch ( uMsg )
	{
		case WM_INITDIALOG:
		{
			// Get instance pointer.
			COptionDialog* pThis = reinterpret_cast<COptionDialog*>( lParam );

			// Initialize the dialog box.
			if ( NULL != pThis )
			{
				// Save instance pointer as window's user data.
				::SetWindowLongPtr( hWnd, DLG_USR3264, lParam );

				pThis->OnInitDialog( hWnd, wParam, lParam );

				nResult = TRUE; // Dialog box procedure should return TRUE ...
			}
			break;
		}
		case WM_COMMAND:
		{
			if ( NULL != pThis )
			{
				pThis->OnCommand( hWnd, wParam, lParam );
				nResult = 0; // Handled
			}
			break;
		}
	}

	return nResult;
}

void COptionDialog::OnInitDialog( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
	CString strHelper;

	// Set window's icon.
	::SendMessage( hWnd, WM_SETICON, (WPARAM)ICON_SMALL, (LPARAM)g_hIcon);

	// Initialize member variables from global settings.
	HDC hDC = ::GetDC( hWnd );

	// RFU: Enable check box if short path usage becomes available.
#ifndef _DEBUG
	::EnableWindow( ::GetDlgItem( hWnd, IDCK_SHORTPATH ), false );
#endif

	::CheckDlgButton( hWnd, IDCK_LARGEICON, g_bLargeIcons ? BST_CHECKED : BST_UNCHECKED );
	::CheckDlgButton( hWnd, IDCK_SHORTPATH, g_bShortPath  ? BST_CHECKED : BST_UNCHECKED );
	::CheckDlgButton( hWnd, IDCK_SYSFONT,   g_bSystemFont ? BST_CHECKED : BST_UNCHECKED );

	// Initialize font name combo box.
	LOGFONT lfMatch = {0};
	HWND hName = ::GetDlgItem( hWnd, IDCB_FONTNAME );
	lfMatch.lfCharSet = g_lfMenuFont.lfCharSet; // Only character set is interesting!
	::EnumFontFamiliesEx
		( hDC
		, &lfMatch
		, (FONTENUMPROC)COptionDialog::EnumFontFamExProc
		, (LPARAM)hName
		, 0 
		);
	::SendMessage( hName, CB_SELECTSTRING, (WPARAM)0, (LPARAM)g_lfMenuFont.lfFaceName );

	// Only numbers are allowed inside combo box's edit control!
	HWND hSize = ::GetDlgItem( hWnd, IDCB_FONTSIZE );
	HWND hEdit = ::FindWindowEx( hSize, NULL, _T("EDIT"), NULL );
	_ASSERT( hEdit != NULL );
	LONG_PTR nStyle = ::GetWindowLongPtr( hEdit, GWL_STYLE );
	nStyle |= ES_NUMBER;
	::SetWindowLongPtr( hEdit, GWL_STYLE, nStyle );

	// Initialize font size combo box.
	int lstSizes[] = { 6, 7, 8, 9, 10, 11, 12, 14, 16 };
	for ( int index=0; index < sizeof(lstSizes)/sizeof(lstSizes[0]); index++ )
	{
		strHelper.Format( _T("%d"), lstSizes[index] );
		::SendMessage( hSize, CB_ADDSTRING, (WPARAM)0, (LPARAM)strHelper.String() );
	}

	// Set combo box's current font size selection.
	strHelper.Format( _T("%d"), -::MulDiv( g_lfMenuFont.lfHeight, 72, ::GetDeviceCaps( hDC, LOGPIXELSY ) ) );
	::SetWindowText( hEdit, strHelper.String() );

	::CheckDlgButton( hWnd, IDCK_BOLD,     (g_lfMenuFont.lfWeight == FW_BOLD) ? BST_CHECKED : BST_UNCHECKED );
	::CheckDlgButton( hWnd, IDCK_ITALIC,    g_lfMenuFont.lfItalic             ? BST_CHECKED : BST_UNCHECKED );
	::CheckDlgButton( hWnd, IDCK_UNDERLINE, g_lfMenuFont.lfUnderline          ? BST_CHECKED : BST_UNCHECKED );

	this->EnableFontGroup( hWnd, !g_bSystemFont );

	this->UpdateFontSampleText( hWnd );

	// Send window into the front.
	::SetForegroundWindow( hWnd );

	// Cleanup
	::ReleaseDC( hWnd, hDC );
}

void COptionDialog::OnCommand( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
	if ( BN_CLICKED == HIWORD( wParam ) )
	{
		if ( IDBN_CLOSE == LOWORD( wParam ) )
		{
			this->SaveData( hWnd );
			::EndDialog( hWnd, IDOK );
		}
		else if ( IDCANCEL == LOWORD( wParam ) )
		{
			::EndDialog( hWnd, IDCANCEL );
		}
		else if ( IDCK_SYSFONT == LOWORD( wParam ) )
		{
			this->EnableFontGroup( hWnd, !IsDlgButtonChecked( hWnd, IDCK_SYSFONT ) );
		}
		else
		{
			switch ( LOWORD( wParam ) ) 
			{
			case IDCK_BOLD:
			case IDCK_ITALIC:
			case IDCK_UNDERLINE:
				this->UpdateFontSampleText( hWnd );
				break;
			}
		}
	}
	else if ( CBN_SELCHANGE == HIWORD( wParam ) )
	{
		if ( IDCB_FONTNAME == LOWORD( wParam ) )
		{
			this->UpdateFontSampleText( hWnd );
		}
		else if ( IDCB_FONTSIZE == LOWORD( wParam ) )
		{
			this->ComboEditChangeHelper( hWnd );
		}
	}
	else if ( CBN_EDITCHANGE == HIWORD( wParam ) )
	{
		if ( IDCB_FONTSIZE == LOWORD( wParam ) )
		{
			this->UpdateFontSampleText( hWnd );
		}
	}
}

void COptionDialog::SaveData( HWND hWnd )
{
	_ASSERT( hWnd != NULL );

	HDC hDC = ::GetDC( hWnd );

	// Save all easy to save values.
	g_bLargeIcons = IsDlgButtonChecked( hWnd, IDCK_LARGEICON ) ? true : false;
	g_bShortPath  = IsDlgButtonChecked( hWnd, IDCK_SHORTPATH ) ? true : false;
	g_bSystemFont = IsDlgButtonChecked( hWnd, IDCK_SYSFONT )   ? true : false;
	
	g_lfMenuFont.lfWeight    = IsDlgButtonChecked( hWnd, IDCK_BOLD ) ? FW_BOLD : FW_NORMAL;
	g_lfMenuFont.lfItalic    = IsDlgButtonChecked( hWnd, IDCK_ITALIC );
	g_lfMenuFont.lfUnderline = IsDlgButtonChecked( hWnd, IDCK_UNDERLINE );
	g_lfMenuFont.lfHeight    = this->GetFontSize( hWnd, hDC ); // Use MulDiv()!

	this->GetFontName( hWnd, g_lfMenuFont.lfFaceName, sizeof(g_lfMenuFont.lfFaceName)/sizeof(TCHAR) );

	// Cleanup.
	::ReleaseDC( hWnd, hDC );
}

void COptionDialog::EnableFontGroup( HWND hWnd, bool bEnable )
{
	::EnableWindow( ::GetDlgItem( hWnd, IDLB_FONTSAMPLE     ), bEnable );
	::EnableWindow( ::GetDlgItem( hWnd, IDCB_FONTNAME  ), bEnable );
	::EnableWindow( ::GetDlgItem( hWnd, IDCB_FONTSIZE  ), bEnable );
	::EnableWindow( ::GetDlgItem( hWnd, IDC_FONTTYPE        ), bEnable );
	::EnableWindow( ::GetDlgItem( hWnd, IDCK_BOLD      ), bEnable );
	::EnableWindow( ::GetDlgItem( hWnd, IDCK_ITALIC    ), bEnable );
	::EnableWindow( ::GetDlgItem( hWnd, IDCK_UNDERLINE ), bEnable );
	::EnableWindow( ::GetDlgItem( hWnd, IDLB_FONTNAME  ), bEnable );
	::EnableWindow( ::GetDlgItem( hWnd, IDLB_FONTSIZE  ), bEnable );
}

int COptionDialog::GetFontSize( HWND hWnd, HDC hDC/*=NULL*/ )
{
	_ASSERT( hWnd != NULL );

	HWND hEdit = ::FindWindowEx( ::GetDlgItem( hWnd, IDCB_FONTSIZE ), NULL, _T("EDIT"), NULL );

	_ASSERT( hEdit != NULL );

	int nLength = ::GetWindowTextLength( hEdit ) + 1; // Terminating zero needed!

	CString strTemp;
	::GetWindowText( hEdit, strTemp.CreateBuffer( nLength ), nLength );
	strTemp.CopyBuffer();

	int nSize = (int)strTemp.Long();

	// Use default font size in case of an invalid value.
	if ( nSize < 6 ) { nSize = 6; }

	if ( hDC == NULL ) // Font size result wanted!
	{
		return nSize;
	}
	else // MulDiv() result wanted!
	{
		return -::MulDiv( nSize, ::GetDeviceCaps( hDC, LOGPIXELSY ), 72 );
	}
}

void COptionDialog::GetFontName( HWND hWnd, TCHAR* szName, int nCount )
{
	_ASSERT( hWnd != NULL );
	_ASSERT( szName != NULL );
	
	// Get current selection.
	LRESULT nSelected = ::SendDlgItemMessage( hWnd, IDCB_FONTNAME, CB_GETCURSEL, (WPARAM)0, (LPARAM)0 );

	if ( CB_ERR != nSelected )
	{
		// Get length of current selection.
		LRESULT nLength = ::SendDlgItemMessage( hWnd, IDCB_FONTNAME, CB_GETLBTEXTLEN, (WPARAM)nSelected, (LPARAM)0 );

		if ( CB_ERR != nLength && nLength <= nCount )
		{
			// Get content of current selection.
			::SendDlgItemMessage( hWnd, IDCB_FONTNAME, CB_GETLBTEXT, (WPARAM)nSelected, (LPARAM)szName );
		}
	}
}

void COptionDialog::UpdateFontSampleText( HWND hWnd )
{
	_ASSERT( hWnd != NULL );

	HWND hSample = ::GetDlgItem( hWnd, IDLB_FONTSAMPLE );

	_ASSERT( hSample != NULL );

	HDC hDC = ::GetDC( hSample );

	LOGFONT lfSample = {0};
	lfSample.lfCharSet   = g_lfMenuFont.lfCharSet;
	lfSample.lfHeight    = this->GetFontSize( hWnd, hDC );
	lfSample.lfWeight    = IsDlgButtonChecked( hWnd, IDCK_BOLD )      ? FW_BOLD : FW_NORMAL;
	lfSample.lfItalic    = IsDlgButtonChecked( hWnd, IDCK_ITALIC )    ? TRUE    : FALSE;
	lfSample.lfUnderline = IsDlgButtonChecked( hWnd, IDCK_UNDERLINE ) ? TRUE    : FALSE;
	this->GetFontName( hWnd, lfSample.lfFaceName, sizeof(lfSample.lfFaceName)/sizeof(TCHAR) );

	// Change displayed text.
	::SetWindowText( hSample, lfSample.lfFaceName );

	// Re-create and set sample font.
	::DeleteObject( this->m_hSampleFont );
	this->m_hSampleFont = ::CreateFontIndirect( &lfSample );
	::SendMessage( hSample, WM_SETFONT, (WPARAM)this->m_hSampleFont, (LPARAM)TRUE );

	// Cleanup.
	::ReleaseDC( hSample, hDC );
}

void COptionDialog::ComboEditChangeHelper( HWND hWnd )
{
	_ASSERT( hWnd != NULL );

	LRESULT nSelected = ::SendDlgItemMessage( hWnd, IDCB_FONTSIZE, CB_GETCURSEL, (WPARAM)0, (LPARAM)0 );

	if ( CB_ERR != nSelected )
	{
		LRESULT nLength = ::SendDlgItemMessage( hWnd, IDCB_FONTSIZE, CB_GETLBTEXTLEN, (WPARAM)nSelected, (LPARAM)0 );

		if ( CB_ERR != nLength )
		{
			CString strTemp;
			nLength = ::SendDlgItemMessage( hWnd, IDCB_FONTSIZE, CB_GETLBTEXT, (WPARAM)nSelected, (LPARAM)strTemp.CreateBuffer( nLength + 1 ) );

			if ( CB_ERR != nLength )
			{
				strTemp.CopyBuffer();

				// Get and save currently typed font size.
				HWND hEdit = ::FindWindowEx( ::GetDlgItem( hWnd, IDCB_FONTSIZE ), NULL, _T("EDIT"), NULL );

				_ASSERT( hEdit != NULL );

				::SetWindowText( hEdit, strTemp.String() );
		
				this->UpdateFontSampleText( hWnd );
			}
		}
	}
}

int CALLBACK COptionDialog::EnumFontFamExProc( ENUMLOGFONTEX* pEnumLF, NEWTEXTMETRICEX* pMetric, DWORD dwType, LPARAM lParam )
{
	HWND hFontNameCombo = reinterpret_cast<HWND>( lParam );

	if ( hFontNameCombo != NULL )
	{
		LPCTSTR szFontName = pEnumLF->elfLogFont.lfFaceName;

		if ( CB_ERR == ::SendMessage( hFontNameCombo, CB_FINDSTRING, (WPARAM)-1, (LPARAM)szFontName ) )
		{
			::SendMessage( hFontNameCombo, CB_ADDSTRING, (WPARAM)0, (LPARAM)szFontName );
		}
		return TRUE; // Continue scanning.
	}
	else
	{
		_ASSERT( false );
		return FALSE; // Stop scanning immediately.
	}
}

// EOF