/*
 * Copyright (C) 2009  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "record.h"

#include <stdio.h>		// Includes STL definitions.

#include "regdlg.h"     // CShowTypeData::ValueToString()

const int CRecord::DEFAULT_SHOW_COMMAND = SW_SHOWMAXIMIZED;
const int CRecord::DEFAULT_ICON_INDEX   = 0;

/********************************************************************************************
**
********************************************************************************************/
CRecord::CRecord()
{
	this->Reset();
}

/********************************************************************************************
**
********************************************************************************************/
CRecord::CRecord( CRecord& source )
{
	this->Copy( source );
}

/********************************************************************************************
**
********************************************************************************************/
CRecord::CRecord( CRecord* pSource )
{
	this->Copy( pSource );
}

/********************************************************************************************
**
********************************************************************************************/
CRecord::~CRecord( VOID )
{
}

/********************************************************************************************
**
********************************************************************************************/
void CRecord::Section( LPCTSTR szFormat, ... )
{
	size_t  length  = 0;
	double  dummy   = 0.0; // Load floating point library, otherwise runtime error R6002 will occur!
	va_list argList;

	if ( szFormat != NULL )
	{
		va_start( argList, szFormat );

		 // _vsctprintf doesn't include terminating '\0'!
		length = _vsctprintf( szFormat, argList ) + 1;

		if ( length > 0 )
		{
			 // _vstprintf_s doesn't include terminating '\0'!
			_vstprintf_s
				( this->m_strSection.CreateBuffer( length )
				, length
				, szFormat
				, argList 
				);

			this->m_strSection.CopyBuffer();
			this->m_strSection.ReleaseBuffer();
		}

		va_end( argList );
	}
}

/********************************************************************************************
**
********************************************************************************************/
void CRecord::Name( LPCTSTR szName )
{
	this->m_strName = szName;
}

/********************************************************************************************
**
********************************************************************************************/
void CRecord::Executable( LPCTSTR szPath )
{
	this->m_strExecutable = szPath;
}

/********************************************************************************************
**
********************************************************************************************/
void CRecord::Parameters( LPCTSTR szParam )
{
	this->m_strParameters = szParam;
}

/********************************************************************************************
**
********************************************************************************************/
void CRecord::Working( LPCTSTR szWorking )
{
	this->m_strWorking = szWorking;
}

/********************************************************************************************
**
********************************************************************************************/
void CRecord::Description( LPCTSTR szDescription )
{
	this->m_strDescription = szDescription;
}

/********************************************************************************************
**
********************************************************************************************/
void CRecord::IconFile( LPCTSTR szIcon )
{
	this->m_strIcon = szIcon;
}

/********************************************************************************************
**
********************************************************************************************/
void CRecord::ShowCommand( int nShow )
{
	this->m_nShow = nShow;
}

/********************************************************************************************
**
********************************************************************************************/
void CRecord::IconIndex( int nIcon )
{
	this->m_nIcon = nIcon;
}

/********************************************************************************************
**
********************************************************************************************/
LPCTSTR CRecord::ShowString()
{
	if ( this->IsSeparator() )
	{
		return _T("Separator");
	}
	else
	{
		// REMARK: This function is not defined as inline because of some conflicts with include file "regdlg.h"
		return CShowTypeData::ValueToString( this->m_nShow );
	}
};

/********************************************************************************************
**
********************************************************************************************/
void CRecord::Reset()
{
	// Reset default values for each member variable.
	this->m_strSection     = _T("");
	this->m_strName        = _T("");
	this->m_strExecutable  = _T("");
	this->m_strParameters  = _T("");
	this->m_strWorking     = _T("");
	this->m_strDescription = _T("");
	this->m_strIcon        = _T("");
	this->m_nShow          = CRecord::DEFAULT_SHOW_COMMAND;
	this->m_nIcon          = CRecord::DEFAULT_ICON_INDEX;
}

/********************************************************************************************
**
********************************************************************************************/
void CRecord::Copy( CRecord& record, bool keepSection/*=false*/ )
{
	if ( !keepSection )
	{
		this->m_strSection = record.m_strSection;
	}
	this->m_strName        = record.m_strName;
	this->m_strExecutable  = record.m_strExecutable;
	this->m_strParameters  = record.m_strParameters;
	this->m_strWorking     = record.m_strWorking;
	this->m_strDescription = record.m_strDescription;
	this->m_strIcon        = record.m_strIcon;
	this->m_nShow          = record.m_nShow;
	this->m_nIcon          = record.m_nIcon;
}

/********************************************************************************************
**
********************************************************************************************/
void CRecord::Copy( CRecord* pRecord, bool keepSection/*=false*/ )
{
	if ( pRecord != NULL )
	{
		this->Copy( *pRecord, keepSection );
	}
}

// EOF