/*
 * Copyright (C) 2009  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#pragma once

#include <windows.h>

#include "string.h"

class CRecord
{
private:

	plexdata::string::CString m_strSection;
	plexdata::string::CString m_strName;
	plexdata::string::CString m_strExecutable;
	plexdata::string::CString m_strParameters;
	plexdata::string::CString m_strWorking;
	plexdata::string::CString m_strDescription;
	plexdata::string::CString m_strIcon;
	int m_nShow;
	int m_nIcon;

public:

	static const int DEFAULT_SHOW_COMMAND;
	static const int DEFAULT_ICON_INDEX;

public:

	CRecord();

	CRecord( CRecord& source);
	
	CRecord( CRecord* pSource );

	virtual ~CRecord();

public: // Getter

	inline LPCTSTR Section()
	{
		return this->m_strSection.String();
	};
	
	inline LPCTSTR Name()
	{
		return this->m_strName.String();
	};
	
	inline LPCTSTR Executable()
	{
		return this->m_strExecutable.String();
	};
	
	inline LPCTSTR Parameters()
	{
		return this->m_strParameters.String();
	};
	
	inline LPCTSTR Working()
	{
		return this->m_strWorking.String();
	};
	
	inline LPCTSTR Description()
	{
		return this->m_strDescription.String();
	};
	
	inline LPCTSTR IconFile()
	{
		return this->m_strIcon.String();
	};

	inline const int ShowCommand()
	{
		return this->m_nShow;
	};

	inline const int IconIndex()
	{
		return this->m_nIcon;
	};

	inline LPCTSTR Separator()
	{
		return _T("@");
	}

	inline bool IsSeparator()
	{
		if ( this->m_strName.Substring( 0, 1 ).Equals( _T("-") ) ) // Old style support!
		{
			return true;
		}
		else
		{
			return this->m_strName.Equals( this->Separator() );
		}
	}

public: // Setter
	
	void Section
		( LPCTSTR szFormat
		, ...  
		);

	void Name
		( LPCTSTR szName
		);

	void Executable
		( LPCTSTR szPath
		);
	
	void Parameters
		( LPCTSTR szParam
		);

	void Working
		( LPCTSTR szWorking
		);

	void Description
		( LPCTSTR szDescription
		);

	void IconFile
		( LPCTSTR szIcon
		);

	void ShowCommand
		( int nShow
		);

	void IconIndex
		( int nIcon
		);

public: // Other stuff

	LPCTSTR ShowString
		(
		);

	void Reset
		(
		);

	void Copy
		( CRecord& record
		, bool     keepSection=false 
		);

	void Copy
		( CRecord* pRecord
		, bool     keepSection=false 
		);
};

// EOF