/*
 * Copyright (C) 2009  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "regdlg.h"
#include <commctrl.h>

#include "winmain.h"
#include "reslocator.h"

#ifdef _WIN64
#define DLG_USR3264 DWLP_USER
#else 
#define DLG_USR3264 DWL_USER
#endif // _WIN64

/******************************************************************************
** CShowTypeData
*******************************************************************************/

CShowTypeData::ENTRY CShowTypeData::LIST[] = {
	{ SW_SHOWDEFAULT,     _T("Default")  },
	{ SW_SHOWNORMAL,      _T("Normal")   },
	{ SW_SHOWMINNOACTIVE, _T("Minimize") },
	{ SW_SHOWMAXIMIZED,   _T("Maximize") },
};

CShowTypeData::CShowTypeData()
{
}

CShowTypeData::~CShowTypeData()
{
}

DWORD CShowTypeData::Count()
{
	return sizeof(LIST)/sizeof(LIST[0]);
}

DWORD CShowTypeData::IndexToValue( DWORD dwIndex )
{
	if ( ( 0 <= dwIndex ) && ( dwIndex < CShowTypeData::Count() ) )
	{
		return LIST[dwIndex].value;
	}
	else
	{
		return -1L;
	}
}

DWORD CShowTypeData::ValueToIndex( DWORD dwValue )
{
	DWORD dwCount = CShowTypeData::Count();
	for ( DWORD dwIndex=0L; dwIndex < dwCount; dwIndex++ )
	{
		if ( dwValue == LIST[dwIndex].value )
		{
			return ( dwIndex );
		}
	}
	return -1L;
}

LPCTSTR CShowTypeData::IndexToString( DWORD dwIndex )
{
	if ( ( 0 <= dwIndex ) && ( dwIndex < CShowTypeData::Count() ) )
	{
		return LIST[dwIndex].label;
	}
	else
	{
		return _T(""); // Do not return NULL!
	}
}

LPCTSTR CShowTypeData::ValueToString( DWORD dwValue )
{
	return CShowTypeData::IndexToString( CShowTypeData::ValueToIndex( dwValue ) );
}

/******************************************************************************
** CRegisterDialog
*******************************************************************************/

CRegisterDialog::CRegisterDialog()
{
}

CRegisterDialog::~CRegisterDialog()
{
}

INT_PTR CRegisterDialog::Show( HINSTANCE hInstance, HWND hParent, CRecord* pData, LPCTSTR szTitle )
{
	m_szTitle = szTitle;

	this->m_cRecord.Reset(); // Reset current record data (intentionally with its section)!

	if ( NULL != pData )
	{
		// The section name is not used AND also not needed inside this dialog box!
		this->m_cRecord.Copy( pData );
	}

	// Display the dialog box.
	return ::DialogBoxParam
		( hInstance
		, MAKEINTRESOURCE( IDD_REGISTER )
		, hParent
		, (DLGPROC)CRegisterDialog::DialogProc
		, (LPARAM)this 
		);
}

CRecord* CRegisterDialog::GetData()
{
	return &this->m_cRecord;
}

INT_PTR CALLBACK CRegisterDialog::DialogProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	INT_PTR nResult = 0;
	CRegisterDialog* pThis = reinterpret_cast<CRegisterDialog*>( ::GetWindowLongPtr( hWnd, DLG_USR3264 ) );

	switch ( uMsg )
	{
		case WM_INITDIALOG:
		{
			// Save the instance pointer.
			CRegisterDialog* pThis = reinterpret_cast<CRegisterDialog*>( lParam );

			// Initialize the dialog box.
			if ( NULL != pThis )
			{
				// Save instance pointer as window's user data.
				::SetWindowLongPtr( hWnd, DLG_USR3264, lParam );

				pThis->OnInitDialog( hWnd, wParam, lParam );

				nResult = TRUE; // Dialog box procedure should return TRUE ...
			}
			break;
		}
		case WM_COMMAND:
		{
			if ( NULL != pThis )
			{
				pThis->OnCommand( hWnd, wParam, lParam );
				nResult = 0; // Handled
			}
			break;
		}
		case WM_VSCROLL:
		case WM_HSCROLL:
		{
			if ( NULL != pThis )
			{
				pThis->OnScroll( hWnd, wParam, lParam );
				nResult = 0; // Handled
			}
			break;
		}
		case WM_DESTROY:
		{
			if ( NULL != pThis )
			{
				pThis->OnDestroy( hWnd, wParam, lParam );
				nResult = 0; // Handled.
			}
			break;
		}
	}

	return nResult;
}

void CRegisterDialog::OnInitDialog( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
	// Set the window's icon.
	::SendMessage( hWnd, WM_SETICON, (WPARAM)ICON_SMALL, (LPARAM)g_hIcon);

	// Initialize the combo box with valid show-types.
	int nCount = CShowTypeData::Count();
	for ( int nIndex=0L; nIndex < nCount; nIndex++ )
	{
		::SendDlgItemMessage
			( hWnd
			, IDCB_SHOWCOMMAND
			, CB_INSERTSTRING
			, (WPARAM)nIndex
			, (LPARAM)CShowTypeData::IndexToString(nIndex)
			);
	}

	// Set dialog box title.
	::SetWindowText( hWnd, this->m_szTitle.String() );

	// Configure special push buttons.
	::SendDlgItemMessage
		( hWnd
		, IDCK_SEPARATOR
		, BM_SETIMAGE
		, (WPARAM)IMAGE_ICON
		, (LPARAM)::LoadImage( g_hInstance, MAKEINTRESOURCE( IDI_DASH ), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR )
		);

	::SendDlgItemMessage
		( hWnd
		, IDBN_SELECTFILE
		, BM_SETIMAGE
		, (WPARAM)IMAGE_ICON
		, (LPARAM)::LoadImage( g_hInstance, MAKEINTRESOURCE( IDI_DOTS ), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR )
		);

	// Load dialog data.
	this->LoadData( hWnd );

	::CheckDlgButton( hWnd, IDCK_SEPARATOR, this->m_cRecord.IsSeparator() ? BST_CHECKED : BST_UNCHECKED );
	
	this->EnableComponents( hWnd, !IsDlgButtonChecked( hWnd, IDCK_SEPARATOR ) );

	// Send window to the top of all other windows.
	::SetForegroundWindow( hWnd );
}

void CRegisterDialog::OnCommand( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
	if ( BN_CLICKED == HIWORD(wParam) )
	{
		if ( IDBN_CLOSE == LOWORD(wParam) )
		{
			this->SaveData( hWnd );

			// Finish dialog window.
			::EndDialog( hWnd, IDOK );
		}
		else if ( IDCANCEL == LOWORD(wParam) )
		{
			// Finish dialog window.
			::EndDialog( hWnd, IDCANCEL );
		}
		else if ( IDBN_SELECTFILE == LOWORD(wParam) )
		{
			TCHAR szFilename[INFOTIPSIZE+1] = _T("");
			OPENFILENAME openData = {0};

			openData.lStructSize = sizeof(OPENFILENAME);
			openData.hwndOwner   = hWnd;
			openData.lpstrFilter = _T("Alle Dateien (*.*)\0*.*\0Programme (*.exe)\0*.exe\0Verknpfungen (*.lnk)\0*.lnk\0PDF-Dokumente (*.pdf)\0*.pdf\0Hilfedateien (*.chm)\0*.chm\0\0");
			openData.lpstrFile   = szFilename;
			openData.nMaxFile    = INFOTIPSIZE;
			openData.Flags       = OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_NODEREFERENCELINKS;

			if ( ::GetOpenFileName( &openData ) )
			{
				CRecord theRecord;
				if ( ResolveData( hWnd, szFilename, theRecord ) )
				{
					this->m_cRecord.Copy( theRecord );
					this->LoadData( hWnd );
				}
			}
		}
		else if ( IDCK_SEPARATOR == LOWORD(wParam) )
		{
			this->EnableComponents( hWnd, !IsDlgButtonChecked( hWnd, IDCK_SEPARATOR ) );
		}
	}
}

void CRegisterDialog::OnScroll( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
	HWND hSender = (HWND)lParam;

	if ( hSender != NULL && ::GetDlgCtrlID( hSender ) == IDC_ICONSPIN )
	{
		if ( LOWORD(wParam) == SB_THUMBPOSITION )
		{
			int nIndex = LOWORD( ::SendMessage( hSender, UDM_GETPOS, (WPARAM)NULL, (LPARAM)NULL ) );
			this->UpdateIcon( hWnd, this->m_cRecord.IconFile(), nIndex );
		}
	}
}

void CRegisterDialog::OnDestroy( HWND hWnd, WPARAM wParam, LPARAM lParam )
{
	// Free special push button resources.
	::DestroyIcon( (HICON)::SendDlgItemMessage( hWnd, IDCK_SEPARATOR, BM_GETIMAGE, (WPARAM)IMAGE_ICON, (LPARAM)0 ) );
	::DestroyIcon( (HICON)::SendDlgItemMessage( hWnd, IDBN_SELECTFILE, BM_GETIMAGE, (WPARAM)IMAGE_ICON, (LPARAM)0 ) );
}

void CRegisterDialog::LoadData( HWND hWnd )
{
	_ASSERT( hWnd != NULL );

	// Set dialog control data whit given initialization data.
	::SendDlgItemMessage( hWnd, IDED_PROGNAME,         WM_SETTEXT,   (WPARAM)NULL, (LPARAM)this->m_cRecord.Name() );
	::SendDlgItemMessage( hWnd, IDED_PATHNAME,         WM_SETTEXT,   (WPARAM)NULL, (LPARAM)this->m_cRecord.Executable() );
	::SendDlgItemMessage( hWnd, IDCB_SHOWCOMMAND, CB_SETCURSEL, (WPARAM)CShowTypeData::ValueToIndex( this->m_cRecord.ShowCommand() ), (LPARAM)NULL );
	::SendDlgItemMessage( hWnd, IDED_PARAMETERS,   WM_SETTEXT,   (WPARAM)NULL, (LPARAM)this->m_cRecord.Parameters() );
	::SendDlgItemMessage( hWnd, IDED_WORKINGDIR,      WM_SETTEXT,   (WPARAM)NULL, (LPARAM)this->m_cRecord.Working() );
	::SendDlgItemMessage( hWnd, IDED_DESCRIPTION,  WM_SETTEXT,   (WPARAM)NULL, (LPARAM)this->m_cRecord.Description() );
	::SendDlgItemMessage( hWnd, IDED_ICONFILE,     WM_SETTEXT,   (WPARAM)NULL, (LPARAM)this->m_cRecord.IconFile() );

	this->LoadIcon
		( hWnd
		, this->m_cRecord.IconFile()
		, this->m_cRecord.IconIndex() 
		);
}

void CRegisterDialog::SaveData( HWND hWnd )
{
	_ASSERT( hWnd != NULL );

	plexdata::string::CString strData;
	size_t nLength = 0L;

	// Reset currently set data (intentionally with section name)!
	this->m_cRecord.Reset();

	if ( IsDlgButtonChecked( hWnd, IDCK_SEPARATOR ) )
	{
		this->m_cRecord.Name( this->m_cRecord.Separator() );
		this->m_cRecord.ShowCommand( -1L );
		this->m_cRecord.IconIndex( -1L );
	}
	else
	{
		// Save the name.
		nLength = ::SendDlgItemMessage( hWnd, IDED_PROGNAME, WM_GETTEXTLENGTH, (WPARAM)0, (LPARAM)0 ) + 1;
		::SendDlgItemMessage( hWnd, IDED_PROGNAME, WM_GETTEXT, (WPARAM)nLength, (LPARAM)strData.CreateBuffer(nLength+1) );
		this->m_cRecord.Name( strData.CopyBuffer() );

		// Save the path.
		nLength = ::SendDlgItemMessage( hWnd, IDED_PATHNAME, WM_GETTEXTLENGTH, (WPARAM)0, (LPARAM)0 ) + 1;
		::SendDlgItemMessage( hWnd, IDED_PATHNAME, WM_GETTEXT, (WPARAM)nLength, (LPARAM)strData.CreateBuffer(nLength+1) );
		this->m_cRecord.Executable( strData.CopyBuffer() );

		// Save new showing state.
		this->m_cRecord.ShowCommand
			( CShowTypeData::IndexToValue
				( (DWORD)::SendDlgItemMessage( hWnd, IDCB_SHOWCOMMAND, CB_GETCURSEL, (WPARAM)NULL, (LPARAM)NULL ) 
				) 
			);

		// Save the parameter(s).
		nLength = ::SendDlgItemMessage( hWnd, IDED_PARAMETERS, WM_GETTEXTLENGTH, (WPARAM)0, (LPARAM)0 ) + 1;
		::SendDlgItemMessage( hWnd, IDED_PARAMETERS, WM_GETTEXT, (WPARAM)nLength, (LPARAM)strData.CreateBuffer(nLength+1) );
		this->m_cRecord.Parameters( strData.CopyBuffer() );

		// Save the working directory.
		nLength = ::SendDlgItemMessage( hWnd, IDED_WORKINGDIR, WM_GETTEXTLENGTH, (WPARAM)0, (LPARAM)0 ) + 1;
		::SendDlgItemMessage( hWnd, IDED_WORKINGDIR, WM_GETTEXT, (WPARAM)nLength, (LPARAM)strData.CreateBuffer(nLength+1) );
		this->m_cRecord.Working( strData.CopyBuffer() );

		// Save the description.
		nLength = ::SendDlgItemMessage( hWnd, IDED_DESCRIPTION, WM_GETTEXTLENGTH, (WPARAM)0, (LPARAM)0 ) + 1;
		::SendDlgItemMessage( hWnd, IDED_DESCRIPTION, WM_GETTEXT, (WPARAM)nLength, (LPARAM)strData.CreateBuffer(nLength+1) );
		this->m_cRecord.Description( strData.CopyBuffer() );

		// Save the icon file.
		nLength = ::SendDlgItemMessage( hWnd, IDED_ICONFILE, WM_GETTEXTLENGTH, (WPARAM)0, (LPARAM)0 ) + 1;
		::SendDlgItemMessage( hWnd, IDED_ICONFILE, WM_GETTEXT, (WPARAM)nLength, (LPARAM)strData.CreateBuffer(nLength+1) );
		this->m_cRecord.IconFile( strData.CopyBuffer() );

		// Save the icon index.
		this->m_cRecord.IconIndex
			( LOWORD( (int)::SendDlgItemMessage
				( hWnd
				, IDC_ICONSPIN
				, UDM_GETPOS
				, (WPARAM)NULL
				, (LPARAM)NULL 
				) )
			);
	}
}

void CRegisterDialog::LoadIcon( HWND hParent, const TCHAR* szResource, int nIndex )
{
	HWND hControl = ::GetDlgItem( hParent, IDC_ICONSPIN );

	_ASSERT( hControl   != NULL );
	_ASSERT( szResource != NULL );

	int nCount = 0;

	// REMARK: Usage of function ::ExtractIcon()...
	//         Yet it's true, function ::ExtractIcon() could be used instead!
	//         But that function does not count number of icons as exact as
	//         the resource locates does. That's the reason why function 
	//         ::ExtractIcon() will not be used!

	// Load module and suppress messages if loading fails.
	HMODULE hModule = ::LoadLibraryEx
		( szResource
		, NULL
		, LOAD_LIBRARY_AS_DATAFILE 
		);

	// Calculate number of availble icon resources.
	plexdata::resources::CResourceLocator resLocator;
	while ( resLocator.FindIconID( hModule, nCount ) )
	{
		nCount++;
	}

	::FreeLibrary( hModule );

	// Setup count information.
	TCHAR szValue[50] = _T("");
	if ( nCount > 0 ) { _stprintf_s( szValue, sizeof(szValue)/sizeof(TCHAR), _T("%d"), nCount ); }
	::SendMessage( ::GetDlgItem( hParent, IDLB_ICONCOUNT ), WM_SETTEXT, (WPARAM)NULL, (LPARAM)szValue );

	// Setup the spin control.
	::SendMessage( hControl, UDM_SETRANGE, (WPARAM)NULL, (LPARAM)nCount - 1 );
	::SendMessage( hControl, UDM_SETPOS,   (WPARAM)NULL, (LPARAM)MAKELONG( (short)nIndex, 0 ) );
	::EnableWindow( hControl, ( nCount > 1 ) ? TRUE : FALSE );

	// Update displayed icon image.
	this->UpdateIcon( hParent, szResource, nIndex );
}

void CRegisterDialog::UpdateIcon( HWND hParent, const TCHAR* szResource, int nIndex )
{
	plexdata::resources::CResourceLocator resLocator;

	HWND hControl = ::GetDlgItem( hParent, IDC_ICONIMAGE );

	_ASSERT( hControl   != NULL );
	_ASSERT( szResource != NULL );

	// Get icon image currently used by the image control.
	HICON hImage = (HICON)::SendMessage( hControl, STM_GETICON, (WPARAM)NULL, (LPARAM)NULL );

	// Destroy previously used icon image if exist.
	if ( hImage != NULL )
	{
		// Delete returned image.
		::DestroyIcon( hImage );

		// Reset icon image currently used by the image control.
		::SendMessage( hControl, STM_SETICON, (WPARAM)NULL, (LPARAM)NULL );
	}

	// Reset text information.
	::SendMessage( ::GetDlgItem( hParent, IDLB_ICONRESID ), WM_SETTEXT, (WPARAM)NULL, (LPARAM)_T("") );
	::SendMessage( ::GetDlgItem( hParent, IDLB_ICONINDEX ), WM_SETTEXT, (WPARAM)NULL, (LPARAM)_T("") );

	// Load module and suppress messages if loading fails.
	HMODULE hModule = ::LoadLibraryEx
		( szResource
		, NULL
		, LOAD_LIBRARY_AS_DATAFILE
		);

	if ( resLocator.FindIconID( hModule, nIndex ) )
	{
		// Set new icon image if loading operation was successful.
		hImage = (HICON)::LoadImage
			( hModule
			, resLocator.GetResourceIDString()
			, IMAGE_ICON
			, 0
			, 0
			, LR_DEFAULTSIZE | LR_DEFAULTCOLOR 
			);

		if ( hImage != NULL )
		{
			::SendMessage( hControl, STM_SETICON, (WPARAM)hImage, (LPARAM)NULL );

			plexdata::string::CString strValue;

			strValue.Format( _T("%d"), nIndex + 1 );
			::SendMessage( ::GetDlgItem( hParent, IDLB_ICONINDEX ), WM_SETTEXT, (WPARAM)NULL, (LPARAM)strValue.String() );

			if ( IS_INTRESOURCE( resLocator.GetResourceIDString() ) )
			{
				strValue.Format( _T("%d"), (int)resLocator.GetResourceIDString() );
			}
			else
			{
				strValue.Format( _T("%s"), resLocator.GetResourceIDString() );
			}
			::SendMessage( ::GetDlgItem( hParent, IDLB_ICONRESID ), WM_SETTEXT, (WPARAM)NULL, (LPARAM)strValue.String() );
		}
	}
	else
	{
		// REMARK: Don't take care about the file extension because I've seen that ...
		//         an executable (exactly the executable ARPPRODUCTICON.EXE which is
		//         part of Nokia's PC Suite) is apparently an icon file instead of
		//         an executable. So try to load an icon from every file type and 
		//         if successful display the result.

		// Set new icon image if its loading was successful.
		hImage = (HICON)::LoadImage
			( NULL
			, szResource
			, IMAGE_ICON
			, 0
			, 0
			, LR_LOADFROMFILE | LR_DEFAULTSIZE | LR_DEFAULTCOLOR 
			);

		if ( hImage != NULL )
		{
			::SendMessage( hControl, STM_SETICON, (WPARAM)hImage, (LPARAM)NULL );
		}
	}

	::FreeLibrary( hModule );
}
void CRegisterDialog::EnableComponents( HWND hWnd, bool bEnable )
{
	HWND hChild = ::GetWindow( hWnd, GW_CHILD | GW_HWNDFIRST );
	while ( hChild != NULL )
	{
		switch ( ::GetDlgCtrlID( hChild ) )
		{
		case IDBN_CLOSE:
		case IDCANCEL:
		case IDCK_SEPARATOR:
			// Intentionally empty
			break;
		default:
			::EnableWindow( hChild, bEnable );
			break;
		}

		// Proceed with next child window
		hChild = ::GetWindow( hChild , /*GW_CHILD | */GW_HWNDNEXT );
	}
}

// EOF