/*
 * Copyright (C) 2009  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#pragma once

#include <windows.h>
#include <tchar.h>

namespace plexdata
{
	namespace resources
	{
		class CResourceLocator
		{
		private:

			volatile long m_nSearchCalls;
			LPTSTR        m_szResourceID;
			LPTSTR        m_szResourceName;
			LPTSTR        m_szResourceType;
			int           m_nSearchIndex;

		public:

			CResourceLocator();

			virtual ~CResourceLocator();

		public:

			inline bool FindIconID( TCHAR* szModule, int nIndex )
			{
				return this->FindResourceIDByIndex( szModule, RT_GROUP_ICON, nIndex );
			}

			inline bool FindIconID( HMODULE hModule, int nIndex )
			{
				return this->FindResourceIDByIndex( hModule, RT_GROUP_ICON, nIndex );
			}

			inline bool FindBitmapID( TCHAR* szModule, int nIndex )
			{
				return this->FindResourceIDByIndex( szModule, RT_BITMAP, nIndex );
			}

			inline bool FindBitmapID( HMODULE hModule, int nIndex )
			{
				return this->FindResourceIDByIndex( hModule, RT_BITMAP, nIndex );
			}

			inline bool FindCursorID( TCHAR* szModule, int nIndex )
			{
				return this->FindResourceIDByIndex( szModule, RT_GROUP_CURSOR, nIndex );
			}

			inline bool FindCursorID( HMODULE hModule, int nIndex )
			{
				return this->FindResourceIDByIndex( hModule, RT_GROUP_CURSOR, nIndex );
			}

			// TODO: More is possible ...

			inline LPCTSTR GetResourceIDString()
			{
				if ( this->m_szResourceName != NULL )
				{
					return this->m_szResourceName;
				}
				else if ( this->m_szResourceID != NULL )
				{
					return MAKEINTRESOURCE( this->m_szResourceID );
				}
				else
				{
					return NULL;
				}
			}

		private:

			bool FindResourceIDByIndex( TCHAR* szModule, LPCTSTR szType, int nIndex );
			bool FindResourceIDByIndex( HMODULE hModule, LPCTSTR szType, int nIndex );

		private:

			// System callbacks.
			static BOOL EnumerateResourceNames( HMODULE hModule, LPCTSTR lpType, LPTSTR lpName, LONG lParam );
			static BOOL EnumerateResourceTypes( HMODULE hModule, LPTSTR lpType, LONG lParam );
		};
	}
}


/// EOF