/*
 * Copyright (C) 2009  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#pragma once

#include <shlobj.h>

namespace plexdata
{
	namespace shelllink
	{
		/*
		 * How to use this shit...
		 * 
		 * To determine path and file name of an executable from an LNK file you need to use 
		 * classes CMsiResolver and CShellResolver like this:
		 * 
		 * 	TCHAR* szShellLink = _T("C:\\Documents and Settings\\All Users\\Desktop\\Word 2003.lnk");
		 * 	TCHAR  szExecutable[INFOTIPSIZE+1] = _T("");
		 * 	TCHAR  szIconpath[INFOTIPSIZE+1] = _T("");
		 * 	int    nIconID = -1;
		 * 	
		 * 	CMsiResolver msiResolver;
		 * 	CShellResolver shellResolver;
		 * 	
		 * 	if ( shellResolver.Initialize() )
		 * 	{
		 * 		if ( shellResolver.Load( szShellLink ) )
		 * 		{
		 * 			bool success = false;
		 * 
		 * 			if ( msiResolver.Initialize() )
		 * 			{
		 * 				if ( msiResolver.IsProperFileType( szShellLink ) )
		 * 				{
		 * 					size_t nExecutable = INFOTIPSIZE;
		 * 					success = msiResolver.GetComponentPath( szShellLink, szExecutable, nExecutable );
		 * 				}
		 * 			}
		 * 
		 * 			if ( !success )
		 * 			{
		 * 				shellResolver.Executable( szExecutable, INFOTIPSIZE );
		 * 			}
		 * 
		 * 			shellResolver.IconInfo( szIconpath, INFOTIPSIZE, nIconID );
		 * 
		 * 			if ( szIconpath[0] == _T('\0') )
		 * 			{
		 * 				_tcscpy_s( szIconpath, INFOTIPSIZE, szExecutable );
		 * 			}
		 * 
		 * 			// More load operations from shell resolver ... 
		 * 		}
		 * 	}
		 *
		 */

		class CMsiResolver
		{
		private:

			// UINT WINAPI MsiGetShortcutTarget(
			//     LPC(W)STR szShortcutPath,   // full file path for the shortcut
			//     LP(W)STR  szProductCode,    // returned product code   - GUID
			//     LP(W)STR  szFeatureId,      // returned Feature Id.
			//     LP(W)STR  szComponentCode); // returned component code - GUID
			typedef UINT ( WINAPI* MsiGetShortcutTargetW_t )( LPCWSTR, LPWSTR, LPWSTR, LPWSTR );
			typedef UINT ( WINAPI* MsiGetShortcutTargetA_t )( LPCSTR,  LPSTR,  LPSTR,  LPSTR  );


			// INSTALLSTATE WINAPI MsiGetComponentPathA(
			//     LPC(W)STR szProduct,        // product code for client product
			//     LPC(W)STR szComponent,      // component Id, string GUID
			//     LP(W)STR  lpPathBuf,        // returned path
			//     LPDWORD   pcchBuf);         // in/out buffer character count
			typedef int /*INSTALLSTATE*/ ( WINAPI* MsiGetComponentPathW_t )( LPCWSTR, LPCWSTR, LPWSTR, LPDWORD );
			typedef int /*INSTALLSTATE*/ ( WINAPI* MsiGetComponentPathA_t )( LPCSTR,  LPCSTR,  LPSTR,  LPDWORD );

		private:

			HMODULE m_hMSIModule;

#ifdef UNICODE
			MsiGetShortcutTargetW_t m_pMsiGetShortcutTarget;
			MsiGetComponentPathW_t  m_pMsiGetComponentPath;
#else
			MsiGetShortcutTargetA_t m_pMsiGetShortcutTarget;
			MsiGetComponentPathA_t  m_pMsiGetComponentPath;
#endif

		public:

			CMsiResolver();

			virtual ~CMsiResolver();

		public:

			bool Initialize();

			bool IsProperFileType
				( __in const TCHAR* szShellLink
				);

			bool GetComponentPath
				( __in    const TCHAR* szShellLink
				, __out   TCHAR*       szPath
				, __inout size_t&      nPath 
				, __in    bool         bShort=false
				);

		};

		class CShellResolver
		{
		private:

			bool          m_bUninitializeNeeded;
			IShellLink*   m_pShellLink;
			IPersistFile* m_pPersistFile;

		public:

			CShellResolver();
			virtual ~CShellResolver();

		public:

			bool Initialize();

			bool Load
				( const TCHAR* szShellLink 
				, HWND         hParent=NULL
				);

			bool Executable
				( TCHAR* szExecutable
				, size_t nExecutable 
				, bool   bShortPath=false
				);

			bool Description
				( TCHAR* szDescription
				, size_t nDescription 
				);

			bool Arguments
				( TCHAR* szArguments
				, size_t nArguments 
				);

			bool Working
				( TCHAR* szWorking
				, size_t nWorking 
				);

			bool IconInfo
				( TCHAR* szIconFile
				, size_t nIconFile
				, int&   nIconID 
				);

			bool ShowCommand
				( int& nShow 
				);
		};
	}
}

// EOF