/*
 * Copyright (C) 2009  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
//#include "stdafx.h"
#include "splitpath.h"
#include <stdio.h>
#include <stdlib.h>

using namespace plexdata::string;

CSplitPath::CSplitPath()
	: m_szFullpath(NULL)
{
	m_szDrive[0]     = _T('\0');
	m_szDirectory[0] = _T('\0');
	m_szFilename[0]  = _T('\0');
	m_szExtension[0] = _T('\0');
}

CSplitPath::CSplitPath( const TCHAR* szPath )
	: m_szFullpath(NULL)
{
	m_szDrive[0]     = _T('\0');
	m_szDirectory[0] = _T('\0');
	m_szFilename[0]  = _T('\0');
	m_szExtension[0] = _T('\0');

	this->Split( szPath );
}

CSplitPath::~CSplitPath()
{
	delete this->m_szFullpath;
}

bool CSplitPath::Drive( const TCHAR* szDrive )
{
	bool success = false;

	this->m_szDrive[0] = _T('\0');

	if ( szDrive != NULL )
	{
		success = ( 0 == _tcscpy_s
			( this->m_szDrive
			, sizeof(this->m_szDrive)/sizeof(TCHAR)
			, szDrive
			));
	}

	return success;
}

bool CSplitPath::Directory( const TCHAR* szDirectory )
{
	bool success = false;

	this->m_szDirectory[0] = _T('\0');

	if ( szDirectory != NULL )
	{
		success = ( 0 == _tcscpy_s
			( this->m_szDirectory
			, sizeof(this->m_szDirectory)/sizeof(TCHAR)
			, szDirectory
			));
	}

	return success;
}

bool CSplitPath::Filename( const TCHAR* szFilename )
{
	bool success = false;

	this->m_szFilename[0] = _T('\0');

	if ( szFilename != NULL )
	{
		success = ( 0 == _tcscpy_s
			( this->m_szFilename
			, sizeof(this->m_szFilename)/sizeof(TCHAR)
			, szFilename
			));
	}

	return success;
}

bool CSplitPath::Extension( const TCHAR* szExtension )
{
	bool success = false;

	this->m_szExtension[0] = _T('\0');

	if ( szExtension != NULL )
	{
		success = ( 0 == _tcscpy_s
			( this->m_szExtension
			, sizeof(this->m_szExtension)/sizeof(TCHAR)
			, szExtension
			));
	}

	return success;
}

bool CSplitPath::Split( const TCHAR* szPath )
{
	bool success = false;

	if ( szPath != NULL )
	{
		success = ( 0 == _tsplitpath_s
			( szPath
			, this->m_szDrive
			, sizeof(this->m_szDrive)/sizeof(TCHAR)
			, this->m_szDirectory
			, sizeof(this->m_szDirectory)/sizeof(TCHAR)
			, this->m_szFilename
			, sizeof(this->m_szFilename)/sizeof(TCHAR)
			, this->m_szExtension
			, sizeof(this->m_szExtension)/sizeof(TCHAR)
			));

		if ( !success )
		{
			this->m_szDrive[0]     = _T('\0');
			this->m_szDirectory[0] = _T('\0');
			this->m_szFilename[0]  = _T('\0');
			this->m_szExtension[0] = _T('\0');
		}
	}

	return success;
}

const TCHAR* CSplitPath::Build( bool quotes/*=false*/ )
{
	delete this->m_szFullpath;
	this->m_szFullpath = NULL;

	size_t nLength = _tcslen( this->m_szDrive )     +
	                 _tcslen( this->m_szDirectory ) +
	                 _tcslen( this->m_szFilename )  +
	                 _tcslen( this->m_szExtension ) +
				     ( quotes ? 2 : 0 );

	this->m_szFullpath = new TCHAR[ nLength + 1 ];

	if ( this->m_szFullpath != NULL )
	{
		this->m_szFullpath[0] = _T('\0');

		int result = _sntprintf_s
			( this->m_szFullpath
			, nLength + 1
			, nLength
			, ( quotes ? _T("\"%s%s%s%s\"") : _T("%s%s%s%s") )
			, this->m_szDrive
			, this->m_szDirectory
			, this->m_szFilename
			, this->m_szExtension
			);

		// Check for error case.
		if ( result != nLength )
		{
			delete this->m_szFullpath;
			this->m_szFullpath = NULL;
		}
	}

	return this->m_szFullpath;
}

const TCHAR* CSplitPath::BuildWithExtension( const TCHAR* szExtension, bool quotes/*=false*/ )
{
	delete this->m_szFullpath;
	this->m_szFullpath = NULL;

	TCHAR* szUsedExtension = _T("");
	if ( szExtension != NULL )
	{
		szUsedExtension = const_cast<TCHAR*>( szExtension );
	}

	size_t nLength = _tcslen( this->m_szDrive )     +
	                 _tcslen( this->m_szDirectory ) +
	                 _tcslen( this->m_szFilename )  +
	                 _tcslen( szUsedExtension )     +
				     ( quotes ? 2 : 0 );

	this->m_szFullpath = new TCHAR[ nLength + 1 ];

	if ( this->m_szFullpath != NULL )
	{
		this->m_szFullpath[0] = _T('\0');

		int result = _sntprintf_s
			( this->m_szFullpath
			, nLength + 1
			, nLength
			, ( quotes ? _T("\"%s%s%s%s\"") : _T("%s%s%s%s") )
			, this->m_szDrive
			, this->m_szDirectory
			, this->m_szFilename
			, szUsedExtension
			);

		// Check for error case.
		if ( result != nLength )
		{
			delete this->m_szFullpath;
			this->m_szFullpath = NULL;
		}
	}

	return this->m_szFullpath;
}
const TCHAR* CSplitPath::BuildPathOnly( bool quotes/*=false*/ )
{
	delete this->m_szFullpath;
	this->m_szFullpath = NULL;

	size_t nLength = _tcslen( this->m_szDrive )     +
	                 _tcslen( this->m_szDirectory ) +
//	                 _tcslen( this->m_szFilename )  +
//	                 _tcslen( this->m_szExtension ) +
				     ( quotes ? 2 : 0 );

	this->m_szFullpath = new TCHAR[ nLength + 1 ];

	if ( this->m_szFullpath != NULL )
	{
		this->m_szFullpath[0] = _T('\0');

		int result = _sntprintf_s
			( this->m_szFullpath
			, nLength + 1
			, nLength
			, ( quotes ? _T("\"%s%s\"") : _T("%s%s") )
			, this->m_szDrive
			, this->m_szDirectory
//			, this->m_szFilename
//			, this->m_szExtension
			);

		// Check for error case.
		if ( result != nLength )
		{
			delete this->m_szFullpath;
			this->m_szFullpath = NULL;
		}
	}

	return this->m_szFullpath;
}


bool CSplitPath::Equals( const TCHAR* szLeft, const TCHAR* szRight, bool ignore )
{
	if ( szLeft == NULL && szRight == NULL )
	{
		return true;
	}
	else if ( szLeft != NULL && szRight != NULL )
	{
		if ( ignore )
		{
			return ( 0 == _tcsicmp( szLeft, szRight ) );
		}
		else
		{
			return ( 0 == _tcscmp( szLeft, szRight ) );
		}
	}
	else
	{
		return false;
	}
}
