/*
 * Copyright (C) 2009  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#pragma once

#include <tchar.h>
#include <stdlib.h>

namespace plexdata
{
	namespace string
	{
		class CSplitPath
		{
		private:

			TCHAR* m_szFullpath;
			TCHAR  m_szDrive[_MAX_DRIVE];
			TCHAR  m_szDirectory[_MAX_DIR];
			TCHAR  m_szFilename[_MAX_FNAME];
			TCHAR  m_szExtension[_MAX_EXT];

		public:

			CSplitPath();

			CSplitPath( const TCHAR* szPath );

			virtual ~CSplitPath();

		public:

			inline const TCHAR* Drive()
			{
				return this->m_szDrive;
			}

			inline const TCHAR* Directory()
			{
				return this->m_szDirectory;
			}

			inline const TCHAR* Filename()
			{
				return this->m_szFilename;
			}

			inline const TCHAR* Extension()
			{
				return this->m_szExtension;
			}

			inline CSplitPath& operator = ( const TCHAR* szPath )
			{
				this->Split( szPath );
				return *this;
			}

			inline bool IsDrive( const TCHAR* szDrive, bool ignore=true )
			{
				return this->Equals( this->m_szDrive, szDrive, ignore );
			}

			inline bool IsDirectory( const TCHAR* szDirectory, bool ignore=true )
			{
				return this->Equals( this->m_szDirectory, szDirectory, ignore );
			}

			inline bool IsFilename( const TCHAR* szFilename, bool ignore=true )
			{
				return this->Equals( this->m_szFilename, szFilename, ignore );
			}

			inline bool IsExtension( const TCHAR* szExtension, bool ignore=true )
			{
				return this->Equals( this->m_szExtension, szExtension, ignore );
			}

		public:

			bool Drive( const TCHAR* szDrive );

			bool Directory( const TCHAR* szDirectory );

			bool Filename( const TCHAR* szFilename );

			bool Extension( const TCHAR* szExtension );

		public:

			bool Split( const TCHAR* szPath );

			const TCHAR* Build( bool quotes=false );
			
			const TCHAR* BuildWithExtension( const TCHAR* szExtension, bool quotes=false );
			
			const TCHAR* BuildPathOnly( bool quotes=false );

		private:

			bool Equals( const TCHAR* szLeft, const TCHAR* szRight, bool ignore );
		};
	}
}